#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Akini Ross (@akinross) <akinross@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_aaa_key_ring
short_description: Manage AAA Key Rings (pki:KeyRing)
description:
- Manage AAA Key Rings on Cisco ACI fabrics.
options:
  name:
    description:
    - The name of the Key Ring.
    type: str
    aliases: [ key_ring, key_ring_name ]
  cloud_tenant:
    description:
    - The name of the cloud tenant.
    - This attribute is only applicable for Cloud APIC.
    type: str
    aliases: [ tenant, tenant_name ]
  description:
    description:
    - The description of the Key Ring.
    type: str
    aliases: [ descr ]
  certificate:
    description:
    - The certificate of the Key Ring.
    - The certificate contains a public key and signed information for verifying the identity.
    type: str
    aliases: [ cert_data, certificate_data, cert ]
  modulus:
    description:
    - The modulus is the length of the key in bits.
    type: int
    choices: [ 512, 1024, 1536, 2048 ]
  certificate_authority:
    description:
    - The name of the Certificate Authority.
    type: str
    aliases: [ cert_authority, cert_authority_name, certificate_authority_name ]
  key:
    description:
    - The private key for the certificate.
    - Providing this option will always result in a change because it is a secure property that cannot be retrieved from APIC.
    type: str
  key_type:
    description:
    - The type of the private key.
    - This attribute is only configurable in ACI versions 6.0(2h) and above.
    type: str
    choices: [ rsa, ecc ]
  ecc_curve:
    description:
    - The curve of the private key.
    - This attribute requires O(key_type=ecc).
    - This attribute is only configurable in ACI versions 6.0(2h) and above.
    type: str
    choices: [ P256, P384, P521, none ]
    aliases: [ curve ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The C(cloud_tenant) used must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) module can be used for this.
seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(pki:KeyRing).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Akini Ross (@akinross)
"""

EXAMPLES = r"""
- name: Create a Key Ring
  cisco.aci.aci_aaa_key_ring:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: example_key_ring
    certificate: '{{ lookup("file", "pki/example_certificate.crt") }}'
    modulus: 2048
    certificate_authority: example_certificate_authority
    key: '{{ lookup("file", "pki/example_key.key") }}'
    state: present
  delegate_to: localhost

- name: Query a Key Ring
  cisco.aci.aci_aaa_key_ring:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: example_key_ring
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all Certificate Authorities
  cisco.aci.aci_aaa_key_ring:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Delete a Key Ring
  cisco.aci.aci_aaa_key_ring:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: example_key_ring
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec
from ansible_collections.cisco.aci.plugins.module_utils.constants import ECC_CURVE


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        name=dict(type="str", aliases=["key_ring", "key_ring_name"]),  # Not required for querying all objects
        cloud_tenant=dict(type="str", aliases=["tenant_name", "tenant"]),
        description=dict(type="str", aliases=["descr"]),
        certificate=dict(type="str", aliases=["cert_data", "certificate_data", "cert"]),
        modulus=dict(type="int", choices=[512, 1024, 1536, 2048]),
        certificate_authority=dict(type="str", aliases=["cert_authority", "cert_authority_name", "certificate_authority_name"]),
        key=dict(type="str", no_log=True),
        key_type=dict(type="str", choices=["rsa", "ecc"]),
        ecc_curve=dict(type="str", choices=["P256", "P384", "P521", "none"], aliases=["curve"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name"]],
            ["state", "present", ["name"]],
        ],
    )

    name = module.params.get("name")
    cloud_tenant = module.params.get("cloud_tenant")
    description = module.params.get("description")
    certificate = module.params.get("certificate")
    modulus = "mod{0}".format(module.params.get("modulus")) if module.params.get("modulus") else None
    certificate_authority = module.params.get("certificate_authority")
    key = module.params.get("key")
    key_type = module.params.get("key_type")
    ecc_curve = ECC_CURVE[module.params.get("ecc_curve")] if module.params.get("ecc_curve") else None
    state = module.params.get("state")
    name_alias = module.params.get("name_alias")

    aci = ACIModule(module)
    aci_class = "pkiKeyRing"
    parent_class = "cloudCertStore" if cloud_tenant else "pkiEp"
    parent_rn = "tn-{0}/certstore".format(cloud_tenant) if cloud_tenant else "userext/pkiext"

    aci.construct_url(
        root_class=dict(
            aci_class=parent_class,
            aci_rn=parent_rn,
        ),
        subclass_1=dict(
            aci_class=aci_class,
            aci_rn="keyring-{0}".format(name),
            module_object=name,
            target_filter={"name": name},
        ),
    )
    aci.get_existing()

    if state == "present":
        class_config = dict(
            name=name,
            descr=description,
            cert=certificate,
            modulus=modulus,
            tp=certificate_authority,
            nameAlias=name_alias,
            key=key,
        )

        if key_type:
            class_config["keyType"] = key_type.upper()

        if ecc_curve:
            class_config["eccCurve"] = ecc_curve

        aci.payload(aci_class=aci_class, class_config=class_config)

        aci.get_diff(aci_class=aci_class, required_properties=dict(name=name) if cloud_tenant else None)

        # Only wrap the payload in parent class if cloud_tenant is set to avoid apic error
        aci.post_config(parent_class=parent_class if cloud_tenant else None)

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2021 Andrii Bilorus <andrii.bilorus@gmail.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_storageshare
version_added: "1.8.0"
short_description: Manage Azure storage file share
description:
    - Create, update or delete a storage file share in existing storage account.
options:
    resource_group:
        description:
            - Name of the resource group to use.
        required: true
        type: str
    name:
        description:
            - Name of the storage file share to delete or create.
        type: str
        required: true
    account_name:
        description:
            - Name of the parent storage account for the storage file share.
        required: true
        type: str
    access_tier:
        description:
            - The access tier determines the price and in some cases also the performance of a file share. TransactionOptimized if not set.
        type: str
        choices:
            - TransactionOptimized
            - Hot
            - Cool
            - Premium
    metadata:
        description:
            - A name-value pair to associate with the container as metadata.
        type: dict
    state:
        description:
            - State of the storage file share. Use 'present' to create or update a storage file share and use 'absent' to delete a file share.
        default: present
        type: str
        choices:
            - absent
            - present
    quota:
        description:
            - The maximum size of the file share, in gigabytes. Must be greater than 0, and less than or equal to 5TB (5120).
              For large file shares, the maximum size is 102400. By default 102400
        type: int
    enabled_protocols:
        description:
            - The authentication protocol that is used for the file share.
            - Can only be specified when creating a share.
        type: str
        choices:
            - SMB
            - NFS
    root_squash:
        description:
            - The property is for NFS share only.
            - The default is C(NoRootSquash).
        type: str
        choices:
            - NoRootSquash
            - RootSquash
            - AllSquash

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Andrii Bilorus (@ewscat)
'''

EXAMPLES = '''
---
- name: Create storage share
  azure_rm_storageshare:
    name: testShare
    resource_group: myResourceGroup
    account_name: testStorageAccount
    state: present
    access_tier: Cool
    quota: 2048
    metadata:
      key1: value1
      key2: value2

- name: Create share with enalbed protocols
  azure_rm_storageshare:
    name: "{{ share_name }}"
    resource_group: "{{ resource_group }}"
    account_name: "{{ storage_account }}"
    access_tier: "{{ access_tier }}"
    root_squash: RootSquash
    enabled_protocols: NFS

- name: Delete storage share
  azure_rm_storageshare:
    name: testShare
    resource_group: myResourceGroup
    account_name: testStorageAccount
    state: absent
'''

RETURN = '''
state:
    description:
        - Facts about the current state of the storage file file share.
    returned: always
    type: complex
    contains:
            id:
                description:
                    - Resource ID of the storage file share
                sample: "/subscriptions/9e700857-1631-4d8a-aed5-908520ede375/resourceGroups/myResourceGroup/providers/Microsoft.Storage/
                         storageAccounts/mystorageaccount/fileServices/default/shares/myshare"
                returned: always
                type: str
            name:
                description:
                    - Name of the file share
                sample: myshare
                returned: always
                type: str
            type:
                description:
                    - The type of the resource
                sample: "Microsoft.Storage/storageAccounts/fileServices/shares"
                returned: always
                type: str
            etag:
                description:
                    - Resource Etag
                sample: "0x8D75E4BA3E275F1"
                returned: always
                type: str
            last_modified_time:
                description:
                    - Returns the date and time the file share was last modified
                sample: "2021-08-23T08:17:35+00:00"
                returned: always
                type: str
            metadata:
                description:
                    - A name-value pair to associate with the file share as metadata
                sample: '{"key1": "value1"}'
                returned: always
                type: dict
            share_quota:
                description:
                    - The maximum size of the file share, in gigabytes
                sample: 102400
                returned: always
                type: int
            access_tier:
                description:
                    - Access tier for specific file share
                sample: 'TransactionOptimized'
                returned: always
                type: str
            access_tier_change_time:
                description:
                    - Indicates the last modification time for file share access tier
                sample: "2021-08-23T08:17:35+00:00"
                returned: always
                type: str
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class Actions:
    '''
    Action list that can be executed with storage file share
    '''
    NoAction, Create, Update, Delete = range(4)


class AzureRMStorageShare(AzureRMModuleBase):
    '''
    Configuration class for an Azure RM Storage file share resource
    '''

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            name=dict(type='str', required=True),
            account_name=dict(type='str', required=True),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            access_tier=dict(type='str', default=None,
                             choices=['TransactionOptimized', 'Hot', 'Cool', 'Premium']),
            quota=dict(type='int', default=None),
            metadata=dict(type='dict', default=None),
            root_squash=dict(type='str', choices=['NoRootSquash', 'RootSquash', 'AllSquash']),
            enabled_protocols=dict(type='str', choices=['SMB', 'NFS']),
        )
        self.results = dict(
            changed=False,
            state=dict()
        )

        self.resource_group = None
        self.name = None
        self.account_name = None
        self.state = None
        self.quota = None
        self.metadata = None
        self.root_squash = None
        self.enabled_protocols = None

        self.to_do = Actions.NoAction

        super(AzureRMStorageShare, self).__init__(self.module_arg_spec,
                                                  supports_check_mode=True,
                                                  supports_tags=False)

    def exec_module(self, **kwargs):
        '''
        Main module execution method
        '''

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        self.log('Fetching storage file share {0}'.format(self.name))
        response = None
        old_response = self.get_share()

        if old_response is None:
            if self.state == "present":
                self.to_do = Actions.Create
        else:
            if self.state == 'absent':
                self.to_do = Actions.Delete
            else:
                self.to_do = Actions.Update

        if self.to_do == Actions.Create:
            self.results['changed'] = True
            if self.check_mode:
                return self.results
            response = self.create_storage_share()
        elif self.to_do == Actions.Delete:
            self.results['changed'] = True
            if self.check_mode:
                return self.results
            response = self.delete_storage_share()
        elif self.to_do == Actions.Update:
            if self.update_needed(old_response):
                self.results['changed'] = True
                if self.check_mode:
                    return self.results
                response = self.update_storage_share(old_response)
            else:
                self.results['changed'] = False
                response = old_response

        if response is not None:
            self.results['state'] = response
        else:
            self.results['state'] = dict()

        return self.results

    def update_needed(self, old_response):
        '''
        Define if storage file share update needed.
        :param old_response: dict with properties of the storage file share
        :return: True if update needed, else False
        '''
        return ((self.access_tier is not None) and (self.access_tier != old_response.get('access_tier')) or
                (self.quota is not None) and (self.quota != old_response.get('share_quota')) or
                (self.metadata is not None) and (self.metadata != old_response.get('metadata')) or
                (self.root_squash is not None) and (self.root_squash != old_response.get('root_squash')) or
                (self.enabled_protocols is not None) and (self.enabled_protocols != old_response.get('enabled_protocols')))

    def get_share(self):
        '''
        Get the properties of the specified Azure Storage file share.
        :return: dict with properties of the storage file share
        '''
        found = False
        try:
            storage_share = self.storage_client.file_shares.get(resource_group_name=self.resource_group,
                                                                account_name=self.account_name,
                                                                share_name=self.name)
            found = True
            self.log("Response : {0}".format(storage_share))
        except ResourceNotFoundError as e:
            self.log("Did not find the storage file share with name {0} : {1}".format(self.name, str(e)))
        return self.storage_share_to_dict(storage_share) if found else None

    def storage_share_to_dict(self, storage_share):
        '''
        Transform Azure RM Storage share object to dictionary
        :param storage_share: contains information about storage file share
        :type storage_share: FileShare
        :return: dict generated from storage_share
        '''
        return dict(
            id=storage_share.id,
            name=storage_share.name,
            type=storage_share.type,
            etag=storage_share.etag.replace('"', ''),
            last_modified_time=storage_share.last_modified_time,
            metadata=storage_share.metadata,
            share_quota=storage_share.share_quota,
            access_tier=storage_share.access_tier,
            access_tier_change_time=storage_share.access_tier_change_time,
            root_squash=storage_share.root_squash,
            enabled_protocols=storage_share.enabled_protocols
        )

    def create_storage_share(self):
        '''
        Method calling the Azure SDK to create storage file share.
        :return: dict with description of the new storage file share
        '''
        self.log("Creating fileshare {0}".format(self.name))
        try:
            file_share = self.storage_models.FileShare(
                access_tier=self.access_tier,
                share_quota=self.quota,
                metadata=self.metadata,
                root_squash=self.root_squash,
                enabled_protocols=self.enabled_protocols,
            )
            self.storage_client.file_shares.create(resource_group_name=self.resource_group,
                                                   account_name=self.account_name,
                                                   share_name=self.name,
                                                   file_share=file_share)
        except Exception as e:
            self.fail("Error creating file share {0} : {1}".format(self.name, str(e)))
        return self.get_share()

    def update_storage_share(self, old_responce):
        '''
        Method calling the Azure SDK to update storage file share.
        :param old_response: dict with properties of the storage file share
        :return: dict with description of the new storage file share
        '''
        self.log("Creating file share {0}".format(self.name))
        file_share = self.storage_models.FileShare(
            access_tier=self.access_tier if self.access_tier else old_responce.get('access_tier'),
            share_quota=self.quota if self.quota else old_responce.get('share_quota'),
            metadata=self.metadata if self.metadata else old_responce.get('metadata'),
            enabled_protocols=self.enabled_protocols if self.enabled_protocols else old_responce.get('enabled_protocols'),
            root_squash=self.root_squash if self.root_squash else old_responce.get('root_squash'),
        )
        try:
            self.storage_client.file_shares.update(resource_group_name=self.resource_group,
                                                   account_name=self.account_name,
                                                   share_name=self.name,
                                                   file_share=file_share)
        except Exception as e:
            self.fail("Error updating file share {0} : {1}".format(self.name, str(e)))
        return self.get_share()

    def delete_storage_share(self):
        '''
        Method calling the Azure SDK to delete storage share.
        :return: object resulting from the original request
        '''
        try:
            self.storage_client.file_shares.delete(resource_group_name=self.resource_group,
                                                   account_name=self.account_name,
                                                   share_name=self.name)
        except Exception as e:
            self.fail("Error deleting file share {0} : {1}".format(self.name, str(e)))
        return self.get_share()


def main():
    AzureRMStorageShare()


if __name__ == '__main__':
    main()



# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is based on
# the config parser from here: https://github.com/emre/storm/blob/master/storm/parsers/ssh_config_parser.py
# Copyright (C) <2013> <Emre Yilmaz>
# SPDX-License-Identifier: MIT

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import os
import re
import traceback
from operator import itemgetter

PARAMIKO_IMPORT_ERROR: str | None
try:
    from paramiko.config import SSHConfig
except ImportError:
    SSHConfig = object  # type: ignore
    HAS_PARAMIKO = False
    PARAMIKO_IMPORT_ERROR = traceback.format_exc()
else:
    HAS_PARAMIKO = True
    PARAMIKO_IMPORT_ERROR = None


class StormConfig(SSHConfig):
    def parse(self, file_obj):
        """
        Read an OpenSSH config from the given file object.
        @param file_obj: a file-like object to read the config file from
        @type file_obj: file
        """
        order = 1
        host = {
            "host": ["*"],
            "config": {},
        }
        for line in file_obj:
            line = line.rstrip("\n").lstrip()
            if line == "":
                self._config.append(
                    {
                        "type": "empty_line",
                        "value": line,
                        "host": "",
                        "order": order,
                    }
                )
                order += 1
                continue

            if line.startswith("#"):
                self._config.append(
                    {
                        "type": "comment",
                        "value": line,
                        "host": "",
                        "order": order,
                    }
                )
                order += 1
                continue

            if "=" in line:
                # Ensure ProxyCommand gets properly split
                if line.lower().strip().startswith("proxycommand"):
                    proxy_re = re.compile(r"^(proxycommand)\s*=*\s*(.*)", re.I)
                    match = proxy_re.match(line)
                    key, value = match.group(1).lower(), match.group(2)
                else:
                    key, value = line.split("=", 1)
                    key = key.strip().lower()
            else:
                # find first whitespace, and split there
                i = 0
                while (i < len(line)) and not line[i].isspace():
                    i += 1
                if i == len(line):
                    raise Exception(f"Unparsable line: {line!r}")
                key = line[:i].lower()
                value = line[i:].lstrip()
            if key == "host":
                self._config.append(host)
                value = value.split()
                host = {key: value, "config": {}, "type": "entry", "order": order}
                order += 1
            elif key in ["identityfile", "localforward", "remoteforward"]:
                if key in host["config"]:
                    host["config"][key].append(value)
                else:
                    host["config"][key] = [value]
            elif key not in host["config"]:
                host["config"].update({key: value})
        self._config.append(host)


class ConfigParser:
    """
    Config parser for ~/.ssh/config files.
    """

    def __init__(self, ssh_config_file=None):
        if not ssh_config_file:
            ssh_config_file = self.get_default_ssh_config_file()

        self.defaults = {}

        self.ssh_config_file = ssh_config_file

        if not os.path.exists(self.ssh_config_file):
            if not os.path.exists(os.path.dirname(self.ssh_config_file)):
                os.makedirs(os.path.dirname(self.ssh_config_file))
            open(self.ssh_config_file, "w+").close()
            os.chmod(self.ssh_config_file, 0o600)

        self.config_data = []

    def get_default_ssh_config_file(self):
        return os.path.expanduser("~/.ssh/config")

    def load(self):
        config = StormConfig()

        with open(self.ssh_config_file) as fd:
            config.parse(fd)

        for entry in config.__dict__.get("_config"):
            if entry.get("host") == ["*"]:
                self.defaults.update(entry.get("config"))

            if entry.get("type") in ["comment", "empty_line"]:
                self.config_data.append(entry)
                continue

            host_item = {
                "host": entry["host"][0],
                "options": entry.get("config"),
                "type": "entry",
                "order": entry.get("order", 0),
            }

            if len(entry["host"]) > 1:
                host_item.update(
                    {
                        "host": " ".join(entry["host"]),
                    }
                )
            # minor bug in paramiko.SSHConfig that duplicates
            # "Host *" entries.
            if entry.get("config") and len(entry.get("config")) > 0:
                self.config_data.append(host_item)

        return self.config_data

    def add_host(self, host, options):
        self.config_data.append(
            {
                "host": host,
                "options": options,
                "order": self.get_last_index(),
            }
        )

        return self

    def update_host(self, host, options, use_regex=False):
        for index, host_entry in enumerate(self.config_data):
            if host_entry.get("host") == host or (use_regex and re.match(host, host_entry.get("host"))):
                if "deleted_fields" in options:
                    deleted_fields = options.pop("deleted_fields")
                    for deleted_field in deleted_fields:
                        del self.config_data[index]["options"][deleted_field]

                self.config_data[index]["options"].update(options)

        return self

    def search_host(self, search_string):
        results = []
        for host_entry in self.config_data:
            if host_entry.get("type") != "entry":
                continue
            if host_entry.get("host") == "*":
                continue

            searchable_information = host_entry.get("host")
            for value in host_entry.get("options").values():
                if isinstance(value, list):
                    value = " ".join(value)
                if isinstance(value, int):
                    value = str(value)

                searchable_information += f" {value}"

            if search_string in searchable_information:
                results.append(host_entry)

        return results

    def delete_host(self, host):
        found = 0
        for index, host_entry in enumerate(self.config_data):
            if host_entry.get("host") == host:
                del self.config_data[index]
                found += 1

        if found == 0:
            raise ValueError("No host found")
        return self

    def delete_all_hosts(self):
        self.config_data = []
        self.write_to_ssh_config()

        return self

    def dump(self):
        if len(self.config_data) < 1:
            return

        file_content = ""
        self.config_data = sorted(self.config_data, key=itemgetter("order"))

        for host_item in self.config_data:
            if host_item.get("type") in ["comment", "empty_line"]:
                file_content += f"{host_item.get('value')}\n"
                continue
            host_item_content = f"Host {host_item.get('host')}\n"
            for key, value in host_item.get("options").items():
                if isinstance(value, list):
                    sub_content = ""
                    for value_ in value:
                        sub_content += f"    {key} {value_}\n"
                    host_item_content += sub_content
                else:
                    host_item_content += f"    {key} {value}\n"
            file_content += host_item_content

        return file_content

    def write_to_ssh_config(self):
        with open(self.ssh_config_file, "w+") as f:
            data = self.dump()
            if data:
                f.write(data)
        return self

    def get_last_index(self):
        last_index = 0
        indexes = []
        for item in self.config_data:
            if item.get("order"):
                indexes.append(item.get("order"))
        if len(indexes) > 0:
            last_index = max(indexes)

        return last_index



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ecs_attribute
version_added: 1.0.0
short_description: manage ecs attributes
description:
  - Create, update or delete ECS container instance attributes.
author:
  - Andrej Svenke (@anryko)
options:
    cluster:
        description:
            - The short name or full Amazon Resource Name (ARN) of the cluster
              that contains the resource to apply attributes.
        required: true
        type: str
    state:
        description:
            - The desired state of the attributes.
        required: false
        default: present
        choices: ['present', 'absent']
        type: str
    attributes:
        description:
            - List of attributes.
        required: true
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name of the attribute. Up to 128 letters (uppercase and lowercase),
                      numbers, hyphens, underscores, and periods are allowed.
                required: true
                type: str
            value:
                description:
                    - The value of the attribute. Up to 128 letters (uppercase and lowercase),
                      numbers, hyphens, underscores, periods, at signs (@), forward slashes, colons,
                      and spaces are allowed.
                required: false
                type: str
    ec2_instance_id:
        description:
            - EC2 instance ID of ECS cluster container instance.
        required: true
        type: str
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Set attributes
  community.aws.ecs_attribute:
    state: present
    cluster: test-cluster
    ec2_instance_id: "{{ ec2_id }}"
    attributes:
      - flavor: test
      - migrated
  delegate_to: localhost

- name: Delete attributes
  community.aws.ecs_attribute:
    state: absent
    cluster: test-cluster
    ec2_instance_id: "{{ ec2_id }}"
    attributes:
      - flavor: test
      - migrated
  delegate_to: localhost
"""

RETURN = r"""
attributes:
    description: attributes
    type: complex
    returned: always
    contains:
        cluster:
            description: cluster name
            type: str
        ec2_instance_id:
            description: ec2 instance id of ecs container instance
            type: str
        attributes:
            description: list of attributes
            type: list
            elements: dict
            contains:
                name:
                    description: name of the attribute
                    type: str
                value:
                    description: value of the attribute
                    returned: if present
                    type: str
"""

try:
    import botocore
    from botocore.exceptions import ClientError
    from botocore.exceptions import EndpointConnectionError
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


class EcsAttributes(object):
    """Handles ECS Cluster Attribute"""

    def __init__(self, module, attributes):
        self.module = module
        self.attributes = attributes if self._validate_attrs(attributes) else self._parse_attrs(attributes)

    def __bool__(self):
        return bool(self.attributes)

    __nonzero__ = __bool__

    def __iter__(self):
        return iter(self.attributes)

    @staticmethod
    def _validate_attrs(attrs):
        return all(tuple(attr.keys()) in (("name", "value"), ("value", "name")) for attr in attrs)

    def _parse_attrs(self, attrs):
        attrs_parsed = []
        for attr in attrs:
            if isinstance(attr, dict):
                if len(attr) != 1:
                    self.module.fail_json(msg=f"Incorrect attribute format - {str(attr)}")
                name, value = list(attr.items())[0]
                attrs_parsed.append({"name": name, "value": value})
            elif isinstance(attr, str):
                attrs_parsed.append({"name": attr, "value": None})
            else:
                self.module.fail_json(msg=f"Incorrect attributes format - {str(attrs)}")

        return attrs_parsed

    def _setup_attr_obj(self, ecs_arn, name, value=None, skip_value=False):
        attr_obj = {"targetType": "container-instance", "targetId": ecs_arn, "name": name}
        if not skip_value and value is not None:
            attr_obj["value"] = value

        return attr_obj

    def get_for_ecs_arn(self, ecs_arn, skip_value=False):
        """
        Returns list of attribute dicts ready to be passed to boto3
        attributes put/delete methods.
        """
        return [self._setup_attr_obj(ecs_arn, skip_value=skip_value, **attr) for attr in self.attributes]

    def diff(self, attrs):
        """
        Returns EcsAttributes Object containing attributes which are present
        in self but are absent in passed attrs (EcsAttributes Object).
        """
        attrs_diff = [attr for attr in self.attributes if attr not in attrs]
        return EcsAttributes(self.module, attrs_diff)


class Ec2EcsInstance(object):
    """Handle ECS Cluster Remote Operations"""

    def __init__(self, module, cluster, ec2_id):
        self.module = module
        self.cluster = cluster
        self.ec2_id = ec2_id

        try:
            self.ecs = module.client("ecs")
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Failed to connect to AWS")

        self.ecs_arn = self._get_ecs_arn()

    def _get_ecs_arn(self):
        try:
            ecs_instances_arns = self.ecs.list_container_instances(cluster=self.cluster)["containerInstanceArns"]
            ec2_instances = self.ecs.describe_container_instances(
                cluster=self.cluster, containerInstances=ecs_instances_arns
            )["containerInstances"]
        except (ClientError, EndpointConnectionError) as e:
            self.module.fail_json(msg=f"Can't connect to the cluster - {str(e)}")

        try:
            ecs_arn = next(inst for inst in ec2_instances if inst["ec2InstanceId"] == self.ec2_id)[
                "containerInstanceArn"
            ]
        except StopIteration:
            self.module.fail_json(msg=f"EC2 instance Id not found in ECS cluster - {str(self.cluster)}")

        return ecs_arn

    def attrs_put(self, attrs):
        """Puts attributes on ECS container instance"""
        try:
            self.ecs.put_attributes(cluster=self.cluster, attributes=attrs.get_for_ecs_arn(self.ecs_arn))
        except ClientError as e:
            self.module.fail_json(msg=str(e))

    def attrs_delete(self, attrs):
        """Deletes attributes from ECS container instance."""
        try:
            self.ecs.delete_attributes(
                cluster=self.cluster, attributes=attrs.get_for_ecs_arn(self.ecs_arn, skip_value=True)
            )
        except ClientError as e:
            self.module.fail_json(msg=str(e))

    def attrs_get_by_name(self, attrs):
        """
        Returns EcsAttributes object containing attributes from ECS container instance with names
        matching to attrs.attributes (EcsAttributes Object).
        """
        attr_objs = [{"targetType": "container-instance", "attributeName": attr["name"]} for attr in attrs]

        try:
            matched_ecs_targets = [
                attr_found
                for attr_obj in attr_objs
                for attr_found in self.ecs.list_attributes(cluster=self.cluster, **attr_obj)["attributes"]
            ]
        except ClientError as e:
            self.module.fail_json(msg=f"Can't connect to the cluster - {str(e)}")

        matched_objs = [target for target in matched_ecs_targets if target["targetId"] == self.ecs_arn]

        results = [{"name": match["name"], "value": match.get("value", None)} for match in matched_objs]

        return EcsAttributes(self.module, results)


def main():
    argument_spec = dict(
        state=dict(required=False, default="present", choices=["present", "absent"]),
        cluster=dict(required=True, type="str"),
        ec2_instance_id=dict(required=True, type="str"),
        attributes=dict(required=True, type="list", elements="dict"),
    )

    required_together = [["cluster", "ec2_instance_id", "attributes"]]

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_together=required_together,
    )

    cluster = module.params["cluster"]
    ec2_instance_id = module.params["ec2_instance_id"]
    attributes = module.params["attributes"]

    conti = Ec2EcsInstance(module, cluster, ec2_instance_id)
    attrs = EcsAttributes(module, attributes)

    results = {
        "changed": False,
        "attributes": [
            {
                "cluster": cluster,
                "ec2_instance_id": ec2_instance_id,
                "attributes": attributes,
            }
        ],
    }

    attrs_present = conti.attrs_get_by_name(attrs)

    if module.params["state"] == "present":
        attrs_diff = attrs.diff(attrs_present)
        if not attrs_diff:
            module.exit_json(**results)

        conti.attrs_put(attrs_diff)
        results["changed"] = True

    elif module.params["state"] == "absent":
        if not attrs_present:
            module.exit_json(**results)

        conti.attrs_delete(attrs_present)
        results["changed"] = True

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2021 Praveen Ghuge (@praveenghuge), Karl Dasan (@ikarldasan)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import absolute_import, division, print_function
__metaclass__ = type
DOCUMENTATION = '''
---
module: azure_rm_ddosprotectionplan
version_added: "1.7.0"
short_description: Manage DDoS protection plan
description:
    - Create, update and delete instance of DDoS protection plan.
options:
    resource_group:
        description:
            - Name of the resource group to which the resource belongs.
        required: true
        type: str
    name:
        description:
            - Unique name of the app service plan to create or update.
        required: true
        type: str
    location:
        description:
            - Resource location. If not set, location from the resource group will be used as default.
        type: str
    state:
        description:
            - Assert the state of the DDoS protection plan.
            - Use C(present) to create or update an DDoS protection plan and C(absent) to delete it.
        type: str
        default: present
        choices:
            - absent
            - present
    log_path:
        description:
            - parent argument.
        type: str
    log_mode:
        description:
            - parent argument.
        type: str
extends_documentation_fragment:
   - azure.azcollection.azure
   - azure.azcollection.azure_tags
author:
    - Praveen Ghuge (@praveenghuge)
    - Karl Dasan (@ikarldasan)
'''
EXAMPLES = '''
- name: "Create DDoS protection plan"
  azure_rm_ddosprotectionplan:
    resource_group: rg
    location: eastus
    name: ddosplan
- name: Delete DDoS protection plan
  azure_rm_ddosprotectionplan:
    resource_group: rg
    name: ddosplan
    state: absent
'''

RETURN = '''
state:
    description:
        - Current state of the DDoS protection plan.
    returned: always
    type: dict
    sample: {
        "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroup/myResourceGroup/providers/Microsoft.Network/ddosProtectionPlans/ddosplan",
        "location": "eastus",
        "name": "ddosplan",
        "etag": "W/60ac0480-44dd-4881-a2ed-680d20b3978e",
        "provisioning_state": "Succeeded",
        "resource_guid": null,
        "type": "Microsoft.Network/ddosProtectionPlans",
        "tags": {"a": "b"},
        "virtual_networks": []
    }
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureDDoSProtectionPlan(AzureRMModuleBase):

    def __init__(self):
        # define user inputs from playbook

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            name=dict(type='str', required=True),
            location=dict(type='str'),
            state=dict(choices=['present', 'absent'],
                       default='present', type='str'),
        )

        self.resource_group = None
        self.name = None
        self.location = None
        self.state = None
        self.tags = None
        self.log_path = None
        self.results = dict(
            changed=False,
            state=dict()
        )

        super(AzureDDoSProtectionPlan, self).__init__(self.module_arg_spec,
                                                      supports_check_mode=True,
                                                      supports_tags=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        self.results['check_mode'] = self.check_mode

        # retrieve resource group to make sure it exists
        self.get_resource_group(self.resource_group)

        results = dict()
        changed = False

        try:
            self.log('Fetching DDoS protection plan {0}'.format(self.name))
            ddos_protection_plan = self.network_client.ddos_protection_plans.get(
                self.resource_group, self.name)

            results = ddos_protection_plan_to_dict(ddos_protection_plan)

            # don't change anything if creating an existing zone, but change if deleting it
            if self.state == 'present':
                changed = False

                update_tags, results['tags'] = self.update_tags(
                    results['tags'])
                if update_tags:
                    changed = True

            elif self.state == 'absent':
                changed = True

        except ResourceNotFoundError:
            # the DDoS protection plan does not exist so create it
            if self.state == 'present':
                changed = True
            else:
                # you can't delete what is not there
                changed = False

        self.results['changed'] = changed
        self.results['state'] = results

        # return the results if you are only gathering information
        if self.check_mode:
            return self.results

        if changed:
            if self.state == "present":
                self.results['state'] = self.create_or_update_ddos_protection_plan(
                    self.module.params)
            elif self.state == "absent":
                # delete DDoS protection plan
                self.delete_ddos_protection_plan()
                self.results['state']['status'] = 'Deleted'

        return self.results

    def create_or_update_ddos_protection_plan(self, params):
        '''
        Create or update DDoS protection plan.
        :return: create or update DDoS protection plan instance state dictionary
        '''
        self.log("create or update DDoS protection plan {0}".format(self.name))
        try:
            poller = self.network_client.ddos_protection_plans.begin_create_or_update(
                resource_group_name=params.get("resource_group"),
                ddos_protection_plan_name=params.get("name"),
                parameters=params)
            result = self.get_poller_result(poller)
            self.log("Response : {0}".format(result))
        except Exception as ex:
            self.fail("Failed to create DDoS protection plan {0} in resource group {1}: {2}".format(
                self.name, self.resource_group, str(ex)))
        return ddos_protection_plan_to_dict(result)

    def delete_ddos_protection_plan(self):
        '''
        Deletes specified DDoS protection plan
        :return True
        '''
        self.log("Deleting the DDoS protection plan {0}".format(self.name))
        try:
            poller = self.network_client.ddos_protection_plans.begin_delete(
                self.resource_group, self.name)
            result = self.get_poller_result(poller)
        except ResourceNotFoundError as e:
            self.log('Error attempting to delete DDoS protection plan.')
            self.fail(
                "Error deleting the DDoS protection plan : {0}".format(str(e)))
        return result


def ddos_protection_plan_to_dict(item):
    # turn DDoS protection plan object into a dictionary (serialization)
    ddos_protection_plan = item.as_dict()

    result = dict(
        additional_properties=ddos_protection_plan.get('additional_properties', None),
        id=ddos_protection_plan.get('id', None),
        name=ddos_protection_plan.get('name', None),
        type=ddos_protection_plan.get('type', None),
        location=ddos_protection_plan.get('location', None),
        tags=ddos_protection_plan.get('tags', None),
        etag=ddos_protection_plan.get('etag', None),
        resource_guid=ddos_protection_plan.get('resource_guid', None),
        provisioning_state=ddos_protection_plan.get('provisioning_state', None),
        virtual_networks=ddos_protection_plan.get('virtual_networks', None)
    )
    return result


def main():
    AzureDDoSProtectionPlan()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["deprecated"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_add_nat_rule
short_description: Create new object.
description:
  - Create new object.
  - All operations are performed over Web Services API.
  - Available from R80 management version.
version_added: "2.0.0"
author: "Or Soffer (@chkp-orso)"
deprecated:
  alternative: cp_mgmt_nat_rule
  why: Newer and updated module released with more functionality.
  removed_at_date: '2024-11-01'
options:
  package:
    description:
      - Name of the package.
    type: str
  position:
    description:
      - Position in the rulebase.
    type: str
  enabled:
    description:
      - Enable/Disable the rule.
    type: bool
  install_on:
    description:
      - Which Gateways identified by the name or UID to install the policy on.
    type: list
    elements: str
  method:
    description:
      - Nat method.
    type: str
    choices: ['static', 'hide', 'nat64', 'nat46']
  original_destination:
    description:
      - Original destination.
    type: str
  original_service:
    description:
      - Original service.
    type: str
  original_source:
    description:
      - Original source.
    type: str
  translated_destination:
    description:
      - Translated  destination.
    type: str
  translated_service:
    description:
      - Translated  service.
    type: str
  translated_source:
    description:
      - Translated  source.
    type: str
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: add-nat-rule
  cp_mgmt_add_nat_rule:
    comments: comment example1 nat999
    enabled: false
    install_on:
      - Policy Targets
    original_destination: All_Internet
    original_source: Any
    package: standard
    position: 1
"""

RETURN = """
cp_mgmt_add_nat_rule:
  description: The checkpoint add-nat-rule output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_commands,
    api_command,
)


def main():
    argument_spec = dict(
        package=dict(type="str"),
        position=dict(type="str"),
        enabled=dict(type="bool"),
        install_on=dict(type="list", elements="str"),
        method=dict(type="str", choices=["static", "hide", "nat64", "nat46"]),
        original_destination=dict(type="str"),
        original_service=dict(type="str"),
        original_source=dict(type="str"),
        translated_destination=dict(type="str"),
        translated_service=dict(type="str"),
        translated_source=dict(type="str"),
        comments=dict(type="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        ignore_warnings=dict(type="bool"),
        ignore_errors=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "add-nat-rule"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# Copyright (c) 2016-2017, Yanis Guenane <yanis+ansible@guenane.org>
# Copyright (c) 2017, Markus Teufelberger <mteufelberger+ansible@mgit.at>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import abc
import typing as t

from ansible_collections.community.crypto.plugins.module_utils._argspec import (
    ArgumentSpec,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLBadPassphraseError,
    OpenSSLObjectError,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.cryptography_support import (
    cryptography_compare_public_keys,
    get_not_valid_after,
    get_not_valid_before,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.certificate_info import (
    get_certificate_info,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.support import (
    load_certificate,
    load_certificate_privatekey,
    load_certificate_request,
)
from ansible_collections.community.crypto.plugins.module_utils._cryptography_dep import (
    COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION,
    assert_required_cryptography_version,
)

if t.TYPE_CHECKING:
    import datetime  # pragma: no cover

    from ansible.module_utils.basic import AnsibleModule  # pragma: no cover

    from ansible_collections.community.crypto.plugins.module_utils._crypto.cryptography_support import (  # pragma: no cover
        CertificatePrivateKeyTypes,
    )


MINIMAL_CRYPTOGRAPHY_VERSION = COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION

try:
    import cryptography
    from cryptography import x509
except ImportError:
    pass


class CertificateError(OpenSSLObjectError):
    pass


class CertificateBackend(metaclass=abc.ABCMeta):
    def __init__(self, *, module: AnsibleModule) -> None:
        self.module = module

        self.force: bool = module.params["force"]
        self.ignore_timestamps: bool = module.params["ignore_timestamps"]
        self.privatekey_path: str | None = module.params["privatekey_path"]
        privatekey_content: str | None = module.params["privatekey_content"]
        if privatekey_content is not None:
            self.privatekey_content: bytes | None = privatekey_content.encode("utf-8")
        else:
            self.privatekey_content = None
        self.privatekey_passphrase: str | None = module.params["privatekey_passphrase"]
        self.csr_path: str | None = module.params["csr_path"]
        csr_content = module.params["csr_content"]
        if csr_content is not None:
            self.csr_content: bytes | None = csr_content.encode("utf-8")
        else:
            self.csr_content = None

        # The following are default values which make sure check() works as
        # before if providers do not explicitly change these properties.
        self.create_subject_key_identifier: str = "never_create"
        self.create_authority_key_identifier: bool = False

        self.privatekey: CertificatePrivateKeyTypes | None = None
        self.csr: x509.CertificateSigningRequest | None = None
        self.cert: x509.Certificate | None = None
        self.existing_certificate: x509.Certificate | None = None
        self.existing_certificate_bytes: bytes | None = None

        self.check_csr_subject: bool = True
        self.check_csr_extensions: bool = True

        self.diff_before = self._get_info(None)
        self.diff_after = self._get_info(None)

    def _get_info(self, data: bytes | None) -> dict[str, t.Any]:
        if data is None:
            return {}
        try:
            result = get_certificate_info(
                module=self.module, content=data, prefer_one_fingerprint=True
            )
            result["can_parse_certificate"] = True
            return result
        except Exception:
            return {"can_parse_certificate": False}

    @abc.abstractmethod
    def generate_certificate(self) -> None:
        """(Re-)Generate certificate."""

    @abc.abstractmethod
    def get_certificate_data(self) -> bytes:
        """Return bytes for self.cert."""

    def set_existing(self, certificate_bytes: bytes | None) -> None:
        """Set existing certificate bytes. None indicates that the key does not exist."""
        self.existing_certificate_bytes = certificate_bytes
        self.diff_after = self.diff_before = self._get_info(
            self.existing_certificate_bytes
        )

    def has_existing(self) -> bool:
        """Query whether an existing certificate is/has been there."""
        return self.existing_certificate_bytes is not None

    def _ensure_private_key_loaded(self) -> None:
        """Load the provided private key into self.privatekey."""
        if self.privatekey is not None:
            return
        if self.privatekey_path is None and self.privatekey_content is None:
            return
        try:
            self.privatekey = load_certificate_privatekey(
                path=self.privatekey_path,
                content=self.privatekey_content,
                passphrase=self.privatekey_passphrase,
            )
        except OpenSSLBadPassphraseError as exc:
            raise CertificateError(exc) from exc

    def _ensure_csr_loaded(self) -> None:
        """Load the CSR into self.csr."""
        if self.csr is not None:
            return
        if self.csr_path is None and self.csr_content is None:
            return
        self.csr = load_certificate_request(
            path=self.csr_path,
            content=self.csr_content,
        )

    def _ensure_existing_certificate_loaded(self) -> None:
        """Load the existing certificate into self.existing_certificate."""
        if self.existing_certificate is not None:
            return
        if self.existing_certificate_bytes is None:
            return
        self.existing_certificate = load_certificate(
            path=None,
            content=self.existing_certificate_bytes,
        )

    def _check_privatekey(self) -> bool:
        """Check whether provided parameters match, assuming self.existing_certificate and self.privatekey have been populated."""
        if self.existing_certificate is None:
            raise AssertionError(  # pragma: no cover
                "Contract violation: existing_certificate has not been populated"
            )
        if self.privatekey is None:
            raise AssertionError(  # pragma: no cover
                "Contract violation: privatekey has not been populated"
            )
        return cryptography_compare_public_keys(
            self.existing_certificate.public_key(), self.privatekey.public_key()
        )

    def _check_csr(self) -> bool:
        """Check whether provided parameters match, assuming self.existing_certificate and self.csr have been populated."""
        if self.existing_certificate is None:
            raise AssertionError(  # pragma: no cover
                "Contract violation: existing_certificate has not been populated"
            )
        if self.csr is None:
            raise AssertionError(
                "Contract violation: csr has not been populated"
            )  # pragma: no cover
        # Verify that CSR is signed by certificate's private key
        if not self.csr.is_signature_valid:
            return False
        if not cryptography_compare_public_keys(
            self.csr.public_key(), self.existing_certificate.public_key()
        ):
            return False
        # Check subject
        if (
            self.check_csr_subject
            and self.csr.subject != self.existing_certificate.subject
        ):
            return False
        # Check extensions
        if not self.check_csr_extensions:
            return True
        cert_exts = list(self.existing_certificate.extensions)
        csr_exts = list(self.csr.extensions)
        if self.create_subject_key_identifier != "never_create":
            # Filter out SubjectKeyIdentifier extension before comparison
            cert_exts = list(
                filter(
                    lambda x: not isinstance(x.value, x509.SubjectKeyIdentifier),
                    cert_exts,
                )
            )
            csr_exts = list(
                filter(
                    lambda x: not isinstance(x.value, x509.SubjectKeyIdentifier),
                    csr_exts,
                )
            )
        if self.create_authority_key_identifier:
            # Filter out AuthorityKeyIdentifier extension before comparison
            cert_exts = list(
                filter(
                    lambda x: not isinstance(x.value, x509.AuthorityKeyIdentifier),
                    cert_exts,
                )
            )
            csr_exts = list(
                filter(
                    lambda x: not isinstance(x.value, x509.AuthorityKeyIdentifier),
                    csr_exts,
                )
            )
        if len(cert_exts) != len(csr_exts):
            return False
        for cert_ext in cert_exts:
            try:
                csr_ext = self.csr.extensions.get_extension_for_oid(cert_ext.oid)
                if cert_ext != csr_ext:
                    return False
            except cryptography.x509.ExtensionNotFound:
                return False
        return True

    def _check_subject_key_identifier(self) -> bool:
        """Check whether Subject Key Identifier matches, assuming self.existing_certificate and self.csr have been populated."""
        if self.existing_certificate is None:
            raise AssertionError(  # pragma: no cover
                "Contract violation: existing_certificate has not been populated"
            )
        if self.csr is None:
            raise AssertionError(
                "Contract violation: csr has not been populated"
            )  # pragma: no cover
        # Get hold of certificate's SKI
        try:
            ext = self.existing_certificate.extensions.get_extension_for_class(
                x509.SubjectKeyIdentifier
            )
        except cryptography.x509.ExtensionNotFound:
            return False
        # Get hold of CSR's SKI for 'create_if_not_provided'
        csr_ext = None
        if self.create_subject_key_identifier == "create_if_not_provided":
            try:
                csr_ext = self.csr.extensions.get_extension_for_class(
                    x509.SubjectKeyIdentifier
                )
            except cryptography.x509.ExtensionNotFound:
                pass
        if csr_ext is None:
            # If CSR had no SKI, or we chose to ignore it ('always_create'), compare with created SKI
            if (
                ext.value.digest
                != x509.SubjectKeyIdentifier.from_public_key(
                    self.existing_certificate.public_key()
                ).digest
            ):
                return False
        else:
            # If CSR had SKI and we did not ignore it ('create_if_not_provided'), compare SKIs
            if ext.value.digest != csr_ext.value.digest:
                return False
        return True

    def needs_regeneration(
        self,
        *,
        not_before: datetime.datetime | None = None,
        not_after: datetime.datetime | None = None,
    ) -> bool:
        """Check whether a regeneration is necessary."""
        if self.force or self.existing_certificate_bytes is None:
            return True

        try:
            self._ensure_existing_certificate_loaded()
        except Exception:
            return True
        assert self.existing_certificate is not None

        # Check whether private key matches
        self._ensure_private_key_loaded()
        if self.privatekey is not None and not self._check_privatekey():
            return True

        # Check whether CSR matches
        self._ensure_csr_loaded()
        if self.csr is not None and not self._check_csr():
            return True

        # Check SubjectKeyIdentifier
        if (
            self.create_subject_key_identifier != "never_create"
            and not self._check_subject_key_identifier()
        ):
            return True

        # Check not before
        if (
            not_before is not None
            and not self.ignore_timestamps
            and get_not_valid_before(self.existing_certificate) != not_before
        ):
            return True

        # Check not after
        return bool(
            not_after is not None
            and not self.ignore_timestamps
            and get_not_valid_after(self.existing_certificate) != not_after
        )

    def dump(self, *, include_certificate: bool) -> dict[str, t.Any]:
        """Serialize the object into a dictionary."""
        result: dict[str, t.Any] = {
            "privatekey": self.privatekey_path,
            "csr": self.csr_path,
        }
        # Get hold of certificate bytes
        certificate_bytes = self.existing_certificate_bytes
        if self.cert is not None:
            certificate_bytes = self.get_certificate_data()
        self.diff_after = self._get_info(certificate_bytes)
        if include_certificate:
            # Store result
            result["certificate"] = (
                certificate_bytes.decode("utf-8") if certificate_bytes else None
            )

        result["diff"] = {
            "before": self.diff_before,
            "after": self.diff_after,
        }
        return result


class CertificateProvider(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def validate_module_args(self, module: AnsibleModule) -> None:
        """Check module arguments"""

    @abc.abstractmethod
    def create_backend(self, module: AnsibleModule) -> CertificateBackend:
        """Create an implementation for a backend.

        Return value must be instance of CertificateBackend.
        """


def select_backend(
    *, module: AnsibleModule, provider: CertificateProvider
) -> CertificateBackend:
    provider.validate_module_args(module)

    assert_required_cryptography_version(
        module, minimum_cryptography_version=MINIMAL_CRYPTOGRAPHY_VERSION
    )

    return provider.create_backend(module)


def get_certificate_argument_spec() -> ArgumentSpec:
    return ArgumentSpec(
        argument_spec={
            "provider": {
                "type": "str",
                "choices": [],
            },  # choices will be filled by add_XXX_provider_to_argument_spec() in certificate_xxx.py
            "force": {
                "type": "bool",
                "default": False,
            },
            "csr_path": {"type": "path"},
            "csr_content": {"type": "str"},
            "ignore_timestamps": {"type": "bool", "default": True},
            "select_crypto_backend": {
                "type": "str",
                "default": "auto",
                "choices": ["auto", "cryptography"],
            },
            # General properties of a certificate
            "privatekey_path": {"type": "path"},
            "privatekey_content": {"type": "str", "no_log": True},
            "privatekey_passphrase": {"type": "str", "no_log": True},
        },
        mutually_exclusive=[
            ["csr_path", "csr_content"],
            ["privatekey_path", "privatekey_content"],
        ],
    )


__all__ = (
    "CertificateBackend",
    "CertificateError",
    "CertificateProvider",
    "get_certificate_argument_spec",
)



# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

PROGRAM_PREFIX = "docker-credential-"
DEFAULT_LINUX_STORE = "secretservice"
DEFAULT_OSX_STORE = "osxkeychain"
DEFAULT_WIN32_STORE = "wincred"



# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# ansible.content_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the documentation in the module file and re-run
# ansible.content_builder commenting out
# the path to external 'docstring' in build.yaml.
#
##############################################

"""
The arg spec for the nxos_l2_interfaces module
"""


class L2_interfacesArgs(object):  # pylint: disable=R0903
    """The arg spec for the nxos_l2_interfaces module"""

    argument_spec = {
        "config": {
            "type": "list",
            "elements": "dict",
            "options": {
                "name": {"type": "str", "required": True},
                "access": {"type": "dict", "options": {"vlan": {"type": "int"}}},
                "trunk": {
                    "type": "dict",
                    "options": {
                        "native_vlan": {"type": "int"},
                        "allowed_vlans": {"type": "str"},
                        "allowed_vlans_none": {"type": "bool"},
                    },
                },
                "mode": {
                    "type": "str",
                    "choices": [
                        "dot1q-tunnel",
                        "access",
                        "trunk",
                        "fex-fabric",
                        "fabricpath",
                    ],
                    "aliases": ["switchport_mode"],
                },
                "cdp_enable": {"type": "bool"},
                "link_flap": {
                    "type": "dict",
                    "options": {
                        "error_disable": {
                            "type": "dict",
                            "options": {
                                "count": {"type": "int"},
                                "interval": {"type": "int"},
                            },
                        },
                    },
                },
                "beacon": {"type": "bool"},
            },
        },
        "state": {
            "type": "str",
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "gathered",
                "rendered",
                "parsed",
            ],
            "default": "merged",
        },
        "running_config": {"type": "str"},
    }  # pylint: disable=C0301



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: backup_tag
version_added: 6.0.0
short_description: Manage tags on backup plan, backup vault, recovery point
description:
    - Create, list, update, remove tags on AWS backup resources such as backup plan, backup vault, and recovery point.
    - Resources are referenced using ARN.
author:
    - Mandar Vijay Kulkarni (@mandar242)
options:
  resource:
    description:
      - The Amazon Resource Name (ARN) of the backup resource.
    required: true
    type: str
  state:
    description:
      - Whether the tags should be present or absent on the resource.
    default: present
    choices: ['present', 'absent']
    type: str
  tags:
    description:
      - A dictionary of tags to add or remove from the resource.
      - If the value provided for a tag key is null and O(state=absent), the tag will be removed regardless of its current value.
    type: dict
    required: true
    aliases: ['resource_tags']
  purge_tags:
    description:
      - Whether unspecified tags should be removed from the resource.
      - Note that when combined with O(state=absent), specified tag keys are not purged regardless of its current value.
    type: bool
    default: false

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Add tags on a resource
  amazon.aws.backup_tag:
    resource: "{{ backup_resource_arn }}"
    state: present
    tags:
      CamelCaseKey: CamelCaseValue
      pascalCaseKey: pascalCaseValue
      snake_case_key: snake_case_value
      test_tag_key_1: tag_tag_value_1
      test_tag_key_2: tag_tag_value_2

- name: Remove only specified tags on a resource
  amazon.aws.backup_tag:
    resource: "{{ backup_resource_arn }}"
    state: absent
    tags:
      CamelCaseKey: CamelCaseValue

- name: Remove all tags except for specified tags
  amazon.aws.backup_tag:
    resource: "{{ backup_resource_arn }}"
    state: absent
    tags:
      test_tag_key_1: tag_tag_value_1
      test_tag_key_2: tag_tag_value_2
    purge_tags: true

- name: Update value of tag key on a resource
  amazon.aws.backup_tag:
    resource: "{{ backup_resource_arn }}"
    state: present
    tags:
      test_tag_key_1: tag_tag_value_NEW_1

- name: Remove all of the tags on a resource
  amazon.aws.backup_tag:
    resource: "{{ backup_resource_arn }}"
    state: absent
    tags: {}
    purge_tags: true
"""

RETURN = r"""
tags:
  description: A dict containing the tags on the resource
  returned: always
  type: dict
added_tags:
  description: A dict of tags that were added to the resource
  returned: When tags are added to the resource
  type: dict
removed_tags:
  description: A dict of tags that were removed from the resource
  returned: When tags are removed from the resource
  type: dict
"""

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible_collections.amazon.aws.plugins.module_utils.backup import get_backup_resource_tags
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags


def manage_tags(module, backup_client):
    result = {"changed": False}

    resource = module.params.get("resource")
    tags = module.params.get("tags")
    state = module.params.get("state")
    purge_tags = module.params.get("purge_tags")

    current_tags = get_backup_resource_tags(module, backup_client, resource)
    tags_to_add, tags_to_remove = compare_aws_tags(current_tags, tags, purge_tags=purge_tags)

    remove_tags = {}
    if state == "absent":
        for key in tags:
            if purge_tags is False and key in current_tags and (tags[key] is None or current_tags[key] == tags[key]):
                remove_tags[key] = current_tags[key]

        for key in tags_to_remove:
            remove_tags[key] = current_tags[key]

        if remove_tags:
            result["changed"] = True
            result["removed_tags"] = remove_tags
            if not module.check_mode:
                try:
                    backup_client.untag_resource(ResourceArn=resource, TagKeyList=list(remove_tags.keys()))
                except (BotoCoreError, ClientError) as remove_tag_error:
                    module.fail_json_aws(
                        remove_tag_error,
                        msg=f"Failed to remove tags {remove_tags} from resource {resource}",
                    )

    if state == "present" and tags_to_add:
        result["changed"] = True
        result["added_tags"] = tags_to_add
        if not module.check_mode:
            try:
                backup_client.tag_resource(ResourceArn=resource, Tags=tags_to_add)
            except (BotoCoreError, ClientError) as set_tag_error:
                module.fail_json_aws(set_tag_error, msg=f"Failed to set tags {tags_to_add} on resource {resource}")

    result["tags"] = get_backup_resource_tags(module, backup_client, resource)
    return result


def main():
    argument_spec = dict(
        state=dict(default="present", choices=["present", "absent"]),
        resource=dict(required=True, type="str"),
        tags=dict(required=True, type="dict", aliases=["resource_tags"]),
        purge_tags=dict(default=False, type="bool"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    backup_client = module.client("backup")

    result = {}

    result = manage_tags(module, backup_client)

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Yunge Zhu (@yungezz)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_virtualnetworkpeering_info
version_added: "0.0.1"
short_description: Get facts of Azure Virtual Network Peering
description:
    - Get facts of Azure Virtual Network Peering.

options:
    resource_group:
        description:
            - Name of a resource group where the vnet exists.
        required: True
        type: str
    virtual_network:
        description:
            - Name or resource ID of a virtual network.
        required: True
        type: raw
    name:
        description:
            - Name of the virtual network peering.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Yunge Zhu (@yungezz)
'''

EXAMPLES = '''
- name: Get virtual network peering by name
  azure_rm_virtualnetworkpeering_info:
    resource_group: myResourceGroup
    virtual_network: myVnet1
    name: myVnetPeer

- name: List virtual network peering of virtual network
  azure_rm_virtualnetworkpeering:
    resource_group: myResourceGroup
    virtual_network: myVnet1
'''

RETURN = '''
vnetpeerings:
    description:
        - A list of Virtual Network Peering facts.
    returned: always
    type: complex
    contains:
        id:
            description: ID of current Virtual Network peering.
            returned: always
            type: str
            sample:
                "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/virtualNetworks/myVnet/virtualNetworkPeerings/peer1"
        name:
            description:
                - Name of Virtual Network peering.
            returned: always
            type: str
            sample: myPeering
        remote_virtual_network:
            description:
                - ID of remote Virtual Network to be peered to.
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/virtualNetworks/myVnet2
        remote_address_space:
            description:
                - The reference of the remote Virtual Network address space.
            type: complex
            returned: always
            contains:
                address_prefixes:
                    description:
                        - A list of address blocks reserved for this Virtual Network in CIDR notation.
                    returned: always
                    type: list
                    sample: 10.1.0.0/16
        peering_state:
            description:
                - The state of the virtual network peering.
            returned: always
            type: str
            sample: Connected
        peering_sync_level:
            description:
                - The Sync Level of the Peering
            type: str
            returned: always
            sample: "FullyInSync"
        provisioning_state:
            description:
                - The provisioning state of the resource.
            returned: always
            type: str
            sample: Succeeded
        allow_forwarded_traffic:
            description:
                - Whether forwarded traffic from the VMs in the remote Virtual Network will be allowed/disallowed.
            returned: always
            type: bool
            sample: False
        allow_gateway_transit:
            description:
                - Whether gateway links can be used in remote Virtual Networking to link to this Virtual Network.
            returned: always
            type: bool
            sample: False
        allow_virtual_network_access:
            description:
                - Whether the VMs in the linked Virtual Network space can access all the VMs in local Virtual Network space.
            returned: always
            type: bool
            sample: False
        use_remote_gateways:
            description:
                - Whether remote gateways can be used on this Virtual Network.
            returned: always
            type: bool
            sample: False
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


def vnetpeering_to_dict(vnetpeering):
    '''
    Convert a virtual network peering object to a dict.
    '''
    results = dict(
        id=vnetpeering.id,
        name=vnetpeering.name,
        remote_virtual_network=vnetpeering.remote_virtual_network.id,
        remote_address_space=dict(
            address_prefixes=vnetpeering.remote_address_space.address_prefixes
        ),
        peering_state=vnetpeering.peering_state,
        provisioning_state=vnetpeering.provisioning_state,
        use_remote_gateways=vnetpeering.use_remote_gateways,
        allow_gateway_transit=vnetpeering.allow_gateway_transit,
        allow_forwarded_traffic=vnetpeering.allow_forwarded_traffic,
        allow_virtual_network_access=vnetpeering.allow_virtual_network_access,
        peering_sync_level=vnetpeering.peering_sync_level
    )
    return results


class AzureRMVirtualNetworkPeeringInfo(AzureRMModuleBase):

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str'
            ),
            virtual_network=dict(
                type='raw',
                required=True
            )
        )

        self.resource_group = None
        self.name = None
        self.virtual_network = None

        self.results = dict(changed=False)

        super(AzureRMVirtualNetworkPeeringInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                               supports_check_mode=True,
                                                               supports_tags=False)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        # parse virtual_network
        self.virtual_network = self.parse_resource_to_dict(self.virtual_network)
        if self.virtual_network['resource_group'] != self.resource_group:
            self.fail('Resource group of virtual_network is not same as param resource_group')

        self.results['vnetpeerings'] = []
        # get vnet peering
        if self.name:
            self.results['vnetpeerings'] = self.get_by_name()
        else:
            self.results['vnetpeerings'] = self.list_by_vnet()

        return self.results

    def get_by_name(self):
        '''
        Gets the Virtual Network Peering.

        :return: List of Virtual Network Peering
        '''
        self.log(
            "Get Virtual Network Peering {0}".format(self.name))
        results = []
        try:
            response = self.network_client.virtual_network_peerings.get(resource_group_name=self.resource_group,
                                                                        virtual_network_name=self.virtual_network['name'],
                                                                        virtual_network_peering_name=self.name)
            self.log("Response : {0}".format(response))
            results.append(vnetpeering_to_dict(response))
        except ResourceNotFoundError:
            self.log('Did not find the Virtual Network Peering.')
        return results

    def list_by_vnet(self):
        '''
        Lists the Virtual Network Peering in specific Virtual Network.

        :return: List of Virtual Network Peering
        '''
        self.log(
            "List Virtual Network Peering in Virtual Network {0}".format(self.virtual_network['name']))
        results = []
        try:
            response = self.network_client.virtual_network_peerings.list(resource_group_name=self.resource_group,
                                                                         virtual_network_name=self.virtual_network['name'])
            self.log("Response : {0}".format(response))
            if response:
                for p in response:
                    results.append(vnetpeering_to_dict(p))
        except ResourceNotFoundError:
            self.log('Did not find the Virtual Network Peering.')
        return results


def main():
    """Main execution"""
    AzureRMVirtualNetworkPeeringInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_ai_storage_policy
short_description: AI-optimized storage policy for Cisco Intersight
description:
  - Creates storage policies optimized for AI workloads including inference model storage,
    training data, and telemetry caches.
  - Provides named storage presets that configure NVMe tiering, drive groups, and
    virtual drive layouts for common AI deployment patterns.
  - Builds on the Intersight storage policy API with AI-specific defaults for
    high-throughput, low-latency access to model weights and datasets.
  - For full storage configuration control, use M(cisco.intersight.intersight_storage_policy) instead.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
    type: str
    default: default
  name:
    description:
      - The name assigned to the storage policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  description:
    description:
      - The user-defined description of the storage policy.
    type: str
    aliases: [descr]
  storage_profile:
    description:
      - The AI storage preset profile to apply.
      - C(inference_optimized) configures NVMe drives in direct-attach mode for lowest latency
        access to model weights. Sets all available NVMe slots to direct-attach, disables
        JBOD for VD creation, and uses RAID0 with read-ahead caching for maximum throughput.
      - C(training_data) configures storage for large dataset access during training.
        Uses RAID5/RAID6 for data protection with write-back caching and larger strip sizes
        for sequential read throughput.
      - C(model_cache) configures a fast NVMe cache tier for model artifacts with M.2 boot.
        Enables M.2 virtual drive for OS boot and sets remaining NVMe drives to direct-attach
        for model weight caching.
      - C(edge_compact) configures storage for Unified Edge nodes with limited drive bays.
        Uses M.2 boot with minimal RAID0 configuration suitable for edge inference with
        constrained storage capacity.
    type: str
    choices: [inference_optimized, training_data, model_cache, edge_compact]
  nvme_slots:
    description:
      - NVMe drive slots to configure in the selected mode.
      - Overrides the default slot configuration from the storage profile.
      - Allowed slots are 1-9, 21-24, 101-104.
      - 'Slot format examples: "1,4,5", "2", "1-5", "1,2,6-8"'
    type: str
  nvme_mode:
    description:
      - Override the NVMe attachment mode.
      - C(direct) attaches NVMe drives directly to the CPU for lowest latency.
      - C(controller) routes NVMe through the RAID controller for management features.
    type: str
    choices: [direct, controller]
  m2_boot_enabled:
    description:
      - Override whether to enable M.2 RAID boot drive.
      - When true, configures M.2 MSTOR-RAID-1 as the boot virtual drive.
    type: bool
  raid_level:
    description:
      - Override the RAID level for data drive groups.
      - Only applicable for training_data profile.
    type: str
    choices: [Raid0, Raid1, Raid5, Raid6, Raid10]
  read_policy:
    description:
      - Override the read cache policy for virtual drives.
    type: str
    choices: [Default, ReadAhead, NoReadAhead]
  write_policy:
    description:
      - Override the write cache policy for virtual drives.
    type: str
    choices: [Default, WriteThrough, WriteBackGoodBbu, AlwaysWriteBack]
author:
  - Steve Fulmer (@stevefulme1)
'''

EXAMPLES = r'''
- name: Create inference-optimized storage policy
  cisco.intersight.intersight_ai_storage_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: AI-Factory
    name: storage-ai-inference
    description: NVMe direct-attach for model weight storage
    storage_profile: inference_optimized

- name: Create training data storage policy with RAID6
  cisco.intersight.intersight_ai_storage_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: storage-ai-training
    description: Protected storage for training datasets
    storage_profile: training_data
    raid_level: Raid6
    write_policy: AlwaysWriteBack

- name: Create model cache storage for inference servers
  cisco.intersight.intersight_ai_storage_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: storage-model-cache
    description: M.2 boot with NVMe model cache
    storage_profile: model_cache
    nvme_slots: "1-4"

- name: Create edge storage policy
  cisco.intersight.intersight_ai_storage_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: storage-edge-inference
    storage_profile: edge_compact

- name: Delete AI storage policy
  cisco.intersight.intersight_ai_storage_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: storage-ai-inference
    storage_profile: inference_optimized
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "storage-ai-inference",
        "ObjectType": "storage.StoragePolicy",
        "DirectAttachedNvmeSlots": "1-4"
    }
applied_settings:
  description: The storage settings applied by the profile and any overrides.
  returned: when state is present
  type: dict
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


# Default NVMe slot ranges
DEFAULT_NVME_SLOTS = '1-4'

# Storage profile presets
STORAGE_PROFILES = {
    'inference_optimized': {
        # All NVMe direct-attach for lowest latency model access
        'DirectAttachedNvmeSlots': DEFAULT_NVME_SLOTS,
        'UseJbodForVdCreation': False,
        'UnusedDisksState': 'NoChange',
        'DefaultDriveMode': 'UnconfiguredGood',
        'M2VirtualDrive': {
            'Enable': False,
            'ControllerSlot': 'MSTOR-RAID-1',
            'Name': 'MStorBootVd',
        },
        'Raid0Drive': {
            'Enable': True,
            'VirtualDrivePolicy': {
                'StripSize': 256,
                'AccessPolicy': 'ReadWrite',
                'ReadPolicy': 'ReadAhead',
                'WritePolicy': 'WriteThrough',
                'DriveCache': 'Enable',
            },
        },
    },
    'training_data': {
        # RAID-protected storage for training datasets
        'UseJbodForVdCreation': False,
        'UnusedDisksState': 'UnconfiguredGood',
        'DefaultDriveMode': 'UnconfiguredGood',
        'M2VirtualDrive': {
            'Enable': True,
            'ControllerSlot': 'MSTOR-RAID-1',
            'Name': 'MStorBootVd',
        },
        'Raid0Drive': {
            'Enable': False,
            'VirtualDrivePolicy': {
                'StripSize': 512,
                'AccessPolicy': 'ReadWrite',
                'ReadPolicy': 'ReadAhead',
                'WritePolicy': 'WriteBackGoodBbu',
                'DriveCache': 'Enable',
            },
        },
        '_raid_level': 'Raid5',
        '_read_policy': 'ReadAhead',
        '_write_policy': 'WriteBackGoodBbu',
    },
    'model_cache': {
        # M.2 boot + NVMe direct-attach model cache
        'DirectAttachedNvmeSlots': DEFAULT_NVME_SLOTS,
        'UseJbodForVdCreation': False,
        'UnusedDisksState': 'NoChange',
        'DefaultDriveMode': 'UnconfiguredGood',
        'M2VirtualDrive': {
            'Enable': True,
            'ControllerSlot': 'MSTOR-RAID-1',
            'Name': 'MStorBootVd',
        },
        'Raid0Drive': {
            'Enable': False,
            'VirtualDrivePolicy': {
                'StripSize': 64,
                'AccessPolicy': 'Default',
                'ReadPolicy': 'Default',
                'WritePolicy': 'Default',
                'DriveCache': 'Default',
            },
        },
    },
    'edge_compact': {
        # Minimal storage for edge nodes
        'UseJbodForVdCreation': False,
        'UnusedDisksState': 'NoChange',
        'DefaultDriveMode': 'UnconfiguredGood',
        'M2VirtualDrive': {
            'Enable': True,
            'ControllerSlot': 'MSTOR-RAID-1',
            'Name': 'MStorBootVd',
        },
        'Raid0Drive': {
            'Enable': True,
            'VirtualDrivePolicy': {
                'StripSize': 64,
                'AccessPolicy': 'ReadWrite',
                'ReadPolicy': 'ReadAhead',
                'WritePolicy': 'WriteThrough',
                'DriveCache': 'Default',
            },
        },
    },
}


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        storage_profile=dict(
            type='str',
            choices=['inference_optimized', 'training_data', 'model_cache', 'edge_compact'],
        ),
        nvme_slots=dict(type='str'),
        nvme_mode=dict(type='str', choices=['direct', 'controller']),
        m2_boot_enabled=dict(type='bool'),
        raid_level=dict(type='str', choices=['Raid0', 'Raid1', 'Raid5', 'Raid6', 'Raid10']),
        read_policy=dict(type='str', choices=['Default', 'ReadAhead', 'NoReadAhead']),
        write_policy=dict(type='str', choices=['Default', 'WriteThrough', 'WriteBackGoodBbu', 'AlwaysWriteBack']),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    resource_path = '/storage/StoragePolicies'

    intersight.api_body = {
        'Name': intersight.module.params['name'],
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
    }

    if module.params['state'] == 'present':
        if not module.params.get('storage_profile'):
            module.fail_json(msg="storage_profile is required when state is 'present'")

        intersight.set_tags_and_description()

        # Deep copy profile settings
        profile_name = module.params['storage_profile']
        import copy
        settings = copy.deepcopy(STORAGE_PROFILES[profile_name])

        # Remove internal metadata keys
        settings.pop('_raid_level', None)
        settings.pop('_read_policy', None)
        settings.pop('_write_policy', None)

        # Apply NVMe slot overrides
        if module.params.get('nvme_slots') is not None:
            nvme_mode = module.params.get('nvme_mode', 'direct')
            if nvme_mode == 'direct':
                settings['DirectAttachedNvmeSlots'] = module.params['nvme_slots']
                settings.pop('ControllerAttachedNvmeSlots', None)
            else:
                settings['ControllerAttachedNvmeSlots'] = module.params['nvme_slots']
                settings.pop('DirectAttachedNvmeSlots', None)
        elif module.params.get('nvme_mode') is not None:
            # Mode override without slot override — move existing slots
            nvme_slots = settings.pop('DirectAttachedNvmeSlots', settings.pop('ControllerAttachedNvmeSlots', None))
            if nvme_slots:
                if module.params['nvme_mode'] == 'direct':
                    settings['DirectAttachedNvmeSlots'] = nvme_slots
                else:
                    settings['ControllerAttachedNvmeSlots'] = nvme_slots

        # Apply M.2 boot override
        if module.params.get('m2_boot_enabled') is not None:
            settings['M2VirtualDrive']['Enable'] = module.params['m2_boot_enabled']

        # Apply cache policy overrides to RAID0 drive config
        if module.params.get('read_policy') is not None:
            settings['Raid0Drive']['VirtualDrivePolicy']['ReadPolicy'] = module.params['read_policy']
        if module.params.get('write_policy') is not None:
            settings['Raid0Drive']['VirtualDrivePolicy']['WritePolicy'] = module.params['write_policy']

        intersight.api_body.update(settings)
        intersight.result['applied_settings'] = settings

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# Copyright (c) 2016-2017, Yanis Guenane <yanis+ansible@guenane.org>
# Copyright (c) 2017, Markus Teufelberger <mteufelberger+ansible@mgit.at>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import os
import tempfile
import traceback
import typing as t

from ansible.module_utils.common.text.converters import to_bytes

from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.certificate import (
    CertificateBackend,
    CertificateError,
    CertificateProvider,
)

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule  # pragma: no cover

    from ansible_collections.community.crypto.plugins.module_utils._argspec import (  # pragma: no cover
        ArgumentSpec,
    )


class AcmeCertificateBackend(CertificateBackend):
    def __init__(self, *, module: AnsibleModule) -> None:
        super().__init__(module=module)
        self.accountkey_path: str = module.params["acme_accountkey_path"]
        self.challenge_path: str = module.params["acme_challenge_path"]
        self.use_chain: bool = module.params["acme_chain"]
        self.acme_directory: str = module.params["acme_directory"]
        self.cert_bytes: bytes | None = None

        if self.csr_content is None:
            if self.csr_path is None:
                raise CertificateError(
                    "csr_path or csr_content is required for ownca provider"
                )
            if not os.path.exists(self.csr_path):
                raise CertificateError(
                    f"The certificate signing request file {self.csr_path} does not exist"
                )

        if not os.path.exists(self.accountkey_path):
            raise CertificateError(
                f"The account key {self.accountkey_path} does not exist"
            )

        if not os.path.exists(self.challenge_path):
            raise CertificateError(
                f"The challenge path {self.challenge_path} does not exist"
            )

        self.acme_tiny_path = self.module.get_bin_path("acme-tiny", required=True)

    def generate_certificate(self) -> None:
        """(Re-)Generate certificate."""

        command = [self.acme_tiny_path]
        if self.use_chain:
            command.append("--chain")
        command.extend(["--account-key", self.accountkey_path])
        if self.csr_content is not None:
            # We need to temporarily write the CSR to disk
            fd, tmpsrc = tempfile.mkstemp()
            self.module.add_cleanup_file(tmpsrc)  # Ansible will delete the file on exit
            f = os.fdopen(fd, "wb")
            try:
                f.write(self.csr_content)
            except Exception as err:
                try:
                    f.close()
                except Exception:
                    pass
                self.module.fail_json(
                    msg=f"failed to create temporary CSR file: {err}",
                    exception=traceback.format_exc(),
                )
            f.close()
            command.extend(["--csr", tmpsrc])
        else:
            assert self.csr_path is not None
            command.extend(["--csr", self.csr_path])
        command.extend(["--acme-dir", self.challenge_path])
        command.extend(["--directory-url", self.acme_directory])

        try:
            self.cert_bytes = to_bytes(
                self.module.run_command(command, check_rc=True)[1]
            )
        except OSError as exc:
            raise CertificateError(exc) from exc

    def get_certificate_data(self) -> bytes:
        """Return bytes for self.cert."""
        if self.cert_bytes is None:
            raise AssertionError(
                "Contract violation: cert_bytes is None"
            )  # pragma: no cover
        return self.cert_bytes

    def dump(self, *, include_certificate: bool) -> dict[str, t.Any]:
        result = super().dump(include_certificate=include_certificate)
        result["accountkey"] = self.accountkey_path
        return result


class AcmeCertificateProvider(CertificateProvider):
    def validate_module_args(self, module: AnsibleModule) -> None:
        if module.params["acme_accountkey_path"] is None:
            module.fail_json(
                msg="The acme_accountkey_path option must be specified for the acme provider."
            )
        if module.params["acme_challenge_path"] is None:
            module.fail_json(
                msg="The acme_challenge_path option must be specified for the acme provider."
            )

    def create_backend(self, module: AnsibleModule) -> AcmeCertificateBackend:
        return AcmeCertificateBackend(module=module)


def add_acme_provider_to_argument_spec(argument_spec: ArgumentSpec) -> None:
    argument_spec.argument_spec["provider"]["choices"].append("acme")  # type: ignore
    argument_spec.argument_spec.update(
        {
            "acme_accountkey_path": {"type": "path"},
            "acme_challenge_path": {"type": "path"},
            "acme_chain": {"type": "bool", "default": False},
            "acme_directory": {
                "type": "str",
                "default": "https://acme-v02.api.letsencrypt.org/directory",
            },
        }
    )


__all__ = (
    "AcmeCertificateBackend",
    "AcmeCertificateProvider",
    "add_acme_provider_to_argument_spec",
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2021, Anvitha Jain (@anvitha-jain) <anvjain@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = r"""
---
module: aci_cloud_bgp_asn
short_description: Manage Cloud APIC BGP Autonomous System Profile (cloud:BgpAsP)
description:
- Manage the BGP Autonomous System Profile for Cloud APIC
author:
- Anvitha Jain (@anvitha-jain)
version_added: '2.1.0'
options:
  asn:
    description:
    - The BGP Autonomous System Number.
    type: str
  description:
    description:
    - Description of the BGP Autonomous System Profile.
    aliases: [ descr ]
    type: str
  name:
    description:
    - object name
    aliases: [ autonomous_system_profile, autonomous_system_profile_name ]
    type: str
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    choices: [ absent, present, query ]
    default: present
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- More information about the internal APIC class B(cloud:BgpAsP) from
  L(the APIC Management Information Model reference,https://developer.cisco.com/docs/apic-mim-ref/).
"""

EXAMPLES = r"""
- name: Add a new cloud BGP ASN
  cisco.aci.aci_cloud_bgp_asn:
    host: apic
    username: admin
    password: SomeSecretPassword
    asn: 64601
    description: ASN description
    name: ASN_1
    state: present
  delegate_to: localhost
- name: Remove a cloud BGP ASN
  cisco.aci.aci_cloud_bgp_asn:
    host: apic
    username: admin
    password: SomeSecretPassword
    validate_certs: false
    state: absent
  delegate_to: localhost
- name: Query a cloud BGP ASN
  cisco.aci.aci_cloud_bgp_asn:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec
from ansible.module_utils.basic import AnsibleModule


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        {
            "asn": dict(type="str"),
            "description": dict(type="str", aliases=["descr"]),
            "name": dict(type="str", aliases=["autonomous_system_profile", "autonomous_system_profile_name"]),
            "name_alias": dict(type="str"),
            "state": dict(type="str", default="present", choices=["absent", "present", "query"]),
        }
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    annotation = module.params["annotation"]
    asn = module.params["asn"]
    description = module.params["description"]
    name = module.params["name"]
    name_alias = module.params["name_alias"]
    state = module.params["state"]
    child_configs = []

    aci = ACIModule(module)
    aci.construct_url(
        root_class={"aci_class": "cloudBgpAsP", "aci_rn": "clouddomp/as", "target_filter": {"name": name}, "module_object": None}, child_classes=[]
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="cloudBgpAsP",
            class_config={
                "annotation": annotation,
                "asn": asn,
                "descr": description,
                "name": name,
                "nameAlias": name_alias,
            },
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="cloudBgpAsP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_external_trusted_ca
short_description: Manages external-trusted-ca objects on Checkpoint over Web Services API
description:
  - Manages external-trusted-ca objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R82 management version.
version_added: "6.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  base64_certificate:
    description:
      - Certificate file encoded in base64.
    type: str
  retrieve_crl_from_http_servers:
    description:
      - Whether to retrieve Certificate Revocation List from http servers.
    type: bool
  crl_cache_method:
    description:
      - Weather to retrieve new Certificate Revocation List after the certificate expires or after a fixed period.
    type: str
    choices: ['timeout', 'expiration date']
  crl_cache_timeout:
    description:
      - When to fetch new Certificate Revocation List (in minutes).
    type: int
  allow_certificates_from_branches:
    description:
      - Allow only certificates from listed branches.
    type: bool
  branches:
    description:
      - Branches to allow certificates from. Required only if "allow-certificates-from-branches" set to "true".
    type: list
    elements: str
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  domains_to_process:
    description:
      - Indicates which domains to process the commands on. It cannot be used with the details-level full, must be run from the System Domain only and
        with ignore-warnings true. Valid values are, CURRENT_DOMAIN, ALL_DOMAINS_ON_THIS_SERVER.
    type: list
    elements: str
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-external-trusted-ca
  cp_mgmt_external_trusted_ca:
    base64_certificate:
      "MIICujCCAaKgAwIBAgIIP1+IHWHbl0EwDQYJKoZIhvcNAQELBQAwFDESMBAGA1UEAxMJd3d3LnouY29tMB4XDTIzMTEyOTEyMzAwMFoXDTI0MTEyMDE2MDAwMFowFDESMBAGA1UEAxMJd3d3LnouY29tMI
       BIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAoBreRGuq8u43GBog+ZaAnaR8ZF8cT2ppvtd3JoFmzTOQivLIt9sNtFYqEgHCtnNkKn9TRrxN14YscHgKIxfDSVlC9Rh0rrBvWgFqcm715Whr99Ogx
       JbYFkusFWJarSejIFx4n6MM48MJxLdtCP6Hy1G2cj1BCiCHj4i3VIVaDE/aMkSqJbYEvf+vFqUWxY8/uEuKI/HGhI7mhUPW4NSGL0Oafz5eEFVsxqV5NA19/JJZ9NajSkyANnaNL5raxGV0oeqaE3JB3lS
       ZfWbH6mQsToUxxwIQfsZiIBozajDdTgP3Kn4SMY0b+I/WAWgfigMSDTAIR8J1sdzGXy2w2kqQIDAQABoxAwDjAMBgNVHRMEBTADAQH/MA0GCSqGSIb3DQEBCwUAA4IBAQBUgrHztHwC1E0mU5c4reMrHg+
       +YRHrgJNHVIYQbL5I2TJHk9S3UZsynoMa1CO86rReOtR5xoGv4PCkyyOW+PNlWUtXF3tNgqWj/21+XzG4RBHPw89TaTxRCdo+MHX58fi07SIzKjmxfdkEi+7+HQEQluDZGViolrGBAw2rXq/SZ3q/11mNq
       b5ZyqyOa2u1sBF1ApvG5a/FBRTaO8gaiNelRf0PGYkuV+1HhF2XyP8Qk565d+uxUH5M7eHF2PNyVk/r/36T+x+UMql9y9iizA0ekuAjXLok1xYl3Vw4S5zXCXYtNZLOVrs+plJb7IrlElyTOAbDFuPugh0
       edz7uZ"
    name: external_ca
    state: present

- name: set-external-trusted-ca
  cp_mgmt_external_trusted_ca:
    base64_certificate:
      "MIICujCCAaKgAwIBAgIIFbLYzT2+3TMwDQYJKoZIhvcNAQELBQAwFDESMBAGA1UEAxMJd3d3LnouY29tMB4XDTI0MDIwMTEyMzEwMFoXDTI0MTIzMTE2MDAwMFowFDESMBAGA1UEAxMJd3d3LnouY29tMI
       BIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAoBreRGuq8u43GBog+ZaAnaR8ZF8cT2ppvtd3JoFmzTOQivLIt9sNtFYqEgHCtnNkKn9TRrxN14YscHgKIxfDSVlC9Rh0rrBvWgFqcm715Whr99Ogx
       JbYFkusFWJarSejIFx4n6MM48MJxLdtCP6Hy1G2cj1BCiCHj4i3VIVaDE/aMkSqJbYEvf+vFqUWxY8/uEuKI/HGhI7mhUPW4NSGL0Oafz5eEFVsxqV5NA19/JJZ9NajSkyANnaNL5raxGV0oeqaE3JB3lS
       ZfWbH6mQsToUxxwIQfsZiIBozajDdTgP3Kn4SMY0b+I/WAWgfigMSDTAIR8J1sdzGXy2w2kqQIDAQABoxAwDjAMBgNVHRMEBTADAQH/MA0GCSqGSIb3DQEBCwUAA4IBAQBxaE9O/LCjKfWeugPeDPvr3Ld
       i1mYsgNIyN+ES1iDoJHXrBQpVzZelJRr8leFgbghGUX7Fwdh1qZ2Jw6nmD1oe/Q7jkPzTngb6dIMI/kFK4eXcS4GJ3S7yGobLB7QUKK1vrYWZdNuAzR6jMRmFECS+lPF7zlTexnwwOkATMp6lzS7xEpEhk
       8eLpSQnYzvsM+rL9voU5q9MrdAJ2XaCZe4Crv75NdYU6ljD2eSYDrO148Tg480TlvT5wzBuyanKhI/Po2oLEVWU7h5tkensHKB5zvxigIr9ZkczdzVbbrRFi2jSQy+VxYWc0zCo/uO+yaKmmLfGDQEb8wZ
       Y1Ml27"
    crl_cache_method: expiration date
    name: external_ca
    retrieve_crl_from_http_servers: 'false'
    state: present

- name: delete-external-trusted-ca
  cp_mgmt_external_trusted_ca:
    name: external_ca
    state: absent
"""

RETURN = """
cp_mgmt_external_trusted_ca:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call


def main():
    argument_spec = dict(
        name=dict(type='str', required=True),
        base64_certificate=dict(type='str'),
        retrieve_crl_from_http_servers=dict(type='bool'),
        crl_cache_method=dict(type='str', choices=['timeout', 'expiration date']),
        crl_cache_timeout=dict(type='int'),
        allow_certificates_from_branches=dict(type='bool'),
        branches=dict(type='list', elements='str'),
        tags=dict(type='list', elements='str'),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        domains_to_process=dict(type='list', elements='str'),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'external-trusted-ca'

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#
# Copyright (c) 2021, Andreas Botzner <andreas at botzner dot com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import traceback
import typing as t

from ansible.module_utils.basic import AnsibleModule, missing_required_lib

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule

REDIS_IMP_ERR: str | None = None
try:
    from redis import Redis
    from redis import __version__ as redis_version

    HAS_REDIS_PACKAGE = True
    REDIS_IMP_ERR = None
except ImportError:
    REDIS_IMP_ERR = traceback.format_exc()
    HAS_REDIS_PACKAGE = False

CERTIFI_IMPORT_ERROR: str | None = None
try:
    import certifi

    HAS_CERTIFI_PACKAGE = True
except ImportError:
    CERTIFI_IMPORT_ERROR = traceback.format_exc()
    HAS_CERTIFI_PACKAGE = False


def fail_imports(module: AnsibleModule, needs_certifi: bool = True) -> None:
    errors: list[str] = []
    traceback: list[str] = []
    if not HAS_REDIS_PACKAGE:
        errors.append(missing_required_lib("redis"))
        traceback.append(REDIS_IMP_ERR)  # type: ignore
    if not HAS_CERTIFI_PACKAGE and needs_certifi:
        errors.append(missing_required_lib("certifi"))
        traceback.append(CERTIFI_IMPORT_ERROR)  # type: ignore
    if errors:
        module.fail_json(msg="\n".join(errors), traceback="\n".join(traceback))


def redis_auth_argument_spec(tls_default: bool = True) -> dict[str, t.Any]:
    return dict(
        login_host=dict(
            type="str",
            default="localhost",
        ),
        login_user=dict(type="str"),
        login_password=dict(type="str", no_log=True),
        login_port=dict(type="int", default=6379),
        tls=dict(type="bool", default=tls_default),
        validate_certs=dict(type="bool", default=True),
        ca_certs=dict(type="str"),
        client_cert_file=dict(type="str"),
        client_key_file=dict(type="str"),
    )


def redis_auth_params(module: AnsibleModule) -> dict[str, t.Any]:
    login_host = module.params["login_host"]
    login_user = module.params["login_user"]
    login_password = module.params["login_password"]
    login_port = module.params["login_port"]
    tls = module.params["tls"]
    validate_certs = "required" if module.params["validate_certs"] else None
    ca_certs = module.params["ca_certs"]
    if tls and ca_certs is None:
        ca_certs = str(certifi.where())
    client_cert_file = module.params["client_cert_file"]
    client_key_file = module.params["client_key_file"]
    if tuple(map(int, redis_version.split("."))) < (3, 4, 0) and login_user is not None:
        module.fail_json(msg="The option `username` in only supported with redis >= 3.4.0.")
    params = {
        "host": login_host,
        "port": login_port,
        "password": login_password,
        "ssl_ca_certs": ca_certs,
        "ssl_certfile": client_cert_file,
        "ssl_keyfile": client_key_file,
        "ssl_cert_reqs": validate_certs,
        "ssl": tls,
    }
    if login_user is not None:
        params["username"] = login_user
    return params


class RedisAnsible:
    """Base class for Redis module"""

    def __init__(self, module: AnsibleModule) -> None:
        self.module = module
        self.connection = self._connect()

    def _connect(self) -> Redis:
        try:
            return Redis(**redis_auth_params(self.module))
        except Exception as e:
            self.module.fail_json(msg=f"{e}")



# Copyright (c) 2021, Florian Dambrine <android.florian@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

"""
Pritunl API that offers CRUD operations on Pritunl Organizations and Users
"""

from __future__ import annotations

import base64
import hashlib
import hmac
import json
import time
import typing as t
import uuid
from http import HTTPStatus

from ansible.module_utils.urls import open_url

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule


class PritunlException(Exception):
    pass


def pritunl_argument_spec() -> dict[str, t.Any]:
    return dict(
        pritunl_url=dict(required=True, type="str"),
        pritunl_api_token=dict(required=True, type="str", no_log=False),
        pritunl_api_secret=dict(required=True, type="str", no_log=True),
        validate_certs=dict(required=False, type="bool", default=True),
    )


def get_pritunl_settings(module: AnsibleModule) -> dict[str, t.Any]:
    """
    Helper function to set required Pritunl request params from module arguments.
    """
    return {
        "api_token": module.params.get("pritunl_api_token"),
        "api_secret": module.params.get("pritunl_api_secret"),
        "base_url": module.params.get("pritunl_url"),
        "validate_certs": module.params.get("validate_certs"),
    }


def _get_pritunl_organizations(api_token, api_secret, base_url: str, validate_certs: bool = True):
    return pritunl_auth_request(
        base_url=base_url,
        api_token=api_token,
        api_secret=api_secret,
        method="GET",
        path="/organization",
        validate_certs=validate_certs,
    )


def _delete_pritunl_organization(
    api_token, api_secret, base_url: str, organization_id: str, validate_certs: bool = True
):
    return pritunl_auth_request(
        base_url=base_url,
        api_token=api_token,
        api_secret=api_secret,
        method="DELETE",
        path=f"/organization/{organization_id}",
        validate_certs=validate_certs,
    )


def _post_pritunl_organization(
    api_token, api_secret, base_url: str, organization_data: object, validate_certs: bool = True
):
    return pritunl_auth_request(
        api_token=api_token,
        api_secret=api_secret,
        base_url=base_url,
        method="POST",
        path="/organization",
        headers={"Content-Type": "application/json"},
        data=json.dumps(organization_data),
        validate_certs=validate_certs,
    )


def _get_pritunl_users(api_token, api_secret, base_url: str, organization_id: str, validate_certs: bool = True):
    return pritunl_auth_request(
        api_token=api_token,
        api_secret=api_secret,
        base_url=base_url,
        method="GET",
        path=f"/user/{organization_id}",
        validate_certs=validate_certs,
    )


def _delete_pritunl_user(
    api_token, api_secret, base_url: str, organization_id: str, user_id: str, validate_certs: bool = True
):
    return pritunl_auth_request(
        api_token=api_token,
        api_secret=api_secret,
        base_url=base_url,
        method="DELETE",
        path=f"/user/{organization_id}/{user_id}",
        validate_certs=validate_certs,
    )


def _post_pritunl_user(
    api_token, api_secret, base_url: str, organization_id: str, user_data: object, validate_certs=True
):
    return pritunl_auth_request(
        api_token=api_token,
        api_secret=api_secret,
        base_url=base_url,
        method="POST",
        path=f"/user/{organization_id}",
        headers={"Content-Type": "application/json"},
        data=json.dumps(user_data),
        validate_certs=validate_certs,
    )


def _put_pritunl_user(
    api_token,
    api_secret,
    base_url: str,
    organization_id: str,
    user_id: str,
    user_data: object,
    validate_certs: bool = True,
):
    return pritunl_auth_request(
        api_token=api_token,
        api_secret=api_secret,
        base_url=base_url,
        method="PUT",
        path=f"/user/{organization_id}/{user_id}",
        headers={"Content-Type": "application/json"},
        data=json.dumps(user_data),
        validate_certs=validate_certs,
    )


def list_pritunl_organizations(
    api_token, api_secret, base_url: str, validate_certs: bool = True, filters: dict[str, t.Any] | None = None
) -> list:
    orgs = []

    response = _get_pritunl_organizations(
        api_token=api_token,
        api_secret=api_secret,
        base_url=base_url,
        validate_certs=validate_certs,
    )

    if response.getcode() != HTTPStatus.OK:
        raise PritunlException("Could not retrieve organizations from Pritunl")
    else:
        for org in json.loads(response.read()):
            # No filtering
            if filters is None:
                orgs.append(org)
            else:
                if not any(filter_val != org[filter_key] for filter_key, filter_val in filters.items()):
                    orgs.append(org)

    return orgs


def list_pritunl_users(
    api_token,
    api_secret,
    base_url: str,
    organization_id: str,
    validate_certs: bool = True,
    filters: dict[str, t.Any] | None = None,
):
    users = []

    response = _get_pritunl_users(
        api_token=api_token,
        api_secret=api_secret,
        base_url=base_url,
        validate_certs=validate_certs,
        organization_id=organization_id,
    )

    if response.getcode() != HTTPStatus.OK:
        raise PritunlException("Could not retrieve users from Pritunl")
    else:
        for user in json.loads(response.read()):
            # No filtering
            if filters is None:
                users.append(user)

            else:
                if not any(filter_val != user[filter_key] for filter_key, filter_val in filters.items()):
                    users.append(user)

    return users


def post_pritunl_organization(
    api_token,
    api_secret,
    base_url: str,
    organization_name: str,
    validate_certs: bool = True,
):
    response = _post_pritunl_organization(
        api_token=api_token,
        api_secret=api_secret,
        base_url=base_url,
        organization_data={"name": organization_name},
        validate_certs=validate_certs,
    )

    if response.getcode() != HTTPStatus.OK:
        raise PritunlException(f"Could not add organization {organization_name} to Pritunl")
    # The user PUT request returns the updated user object
    return json.loads(response.read())


def post_pritunl_user(
    api_token,
    api_secret,
    base_url: str,
    organization_id: str,
    user_data: object,
    user_id: str | None = None,
    validate_certs: bool = True,
):
    # If user_id is provided will do PUT otherwise will do POST
    if user_id is None:
        response = _post_pritunl_user(
            api_token=api_token,
            api_secret=api_secret,
            base_url=base_url,
            organization_id=organization_id,
            user_data=user_data,
            validate_certs=validate_certs,
        )

        if response.getcode() != HTTPStatus.OK:
            raise PritunlException(f"Could not remove user {user_id} from organization {organization_id} from Pritunl")
        # user POST request returns an array of a single item,
        # so return this item instead of the list
        return json.loads(response.read())[0]
    else:
        response = _put_pritunl_user(
            api_token=api_token,
            api_secret=api_secret,
            base_url=base_url,
            organization_id=organization_id,
            user_data=user_data,
            user_id=user_id,
            validate_certs=validate_certs,
        )

        if response.getcode() != HTTPStatus.OK:
            raise PritunlException(f"Could not update user {user_id} from organization {organization_id} from Pritunl")
        # The user PUT request returns the updated user object
        return json.loads(response.read())


def delete_pritunl_organization(
    api_token, api_secret, base_url: str, organization_id: str, validate_certs: bool = True
):
    response = _delete_pritunl_organization(
        api_token=api_token,
        api_secret=api_secret,
        base_url=base_url,
        organization_id=organization_id,
        validate_certs=validate_certs,
    )

    if response.getcode() != HTTPStatus.OK:
        raise PritunlException(f"Could not remove organization {organization_id} from Pritunl")

    return json.loads(response.read())


def delete_pritunl_user(
    api_token, api_secret, base_url: str, organization_id: str, user_id: str, validate_certs: bool = True
):
    response = _delete_pritunl_user(
        api_token=api_token,
        api_secret=api_secret,
        base_url=base_url,
        organization_id=organization_id,
        user_id=user_id,
        validate_certs=validate_certs,
    )

    if response.getcode() != HTTPStatus.OK:
        raise PritunlException(f"Could not remove user {user_id} from organization {organization_id} from Pritunl")

    return json.loads(response.read())


def pritunl_auth_request(
    api_token,
    api_secret,
    base_url: str,
    method: str,
    path: str,
    validate_certs: bool = True,
    headers: dict[str, str] | None = None,
    data: bytes | str | None = None,
):
    """
    Send an API call to a Pritunl server.
    Taken from https://pritunl.com/api and adapted to work with Ansible open_url
    """
    auth_timestamp = str(int(time.time()))
    auth_nonce = uuid.uuid4().hex

    auth_string = f"{api_token}&{auth_timestamp}&{auth_nonce}&{method.upper()}&{path}"

    auth_signature = base64.b64encode(
        hmac.new(api_secret.encode("utf-8"), auth_string.encode("utf-8"), hashlib.sha256).digest()
    )

    auth_headers = {
        "Auth-Token": api_token,
        "Auth-Timestamp": auth_timestamp,
        "Auth-Nonce": auth_nonce,
        "Auth-Signature": auth_signature,
    }

    if headers:
        auth_headers.update(headers)

    try:
        uri = f"{base_url}{path}"

        return open_url(
            uri,
            method=method.upper(),
            headers=auth_headers,
            data=data,
            validate_certs=validate_certs,
        )
    except Exception as e:
        raise PritunlException(e) from e



# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#
# Note: This code should probably live in amazon.aws rather than community.aws.
# However, for the sake of getting something into a useful shape first, it makes
# sense for it to start life in community.aws.
#

from copy import deepcopy
from functools import wraps

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict


class BaseWaiterFactory:
    """
    A helper class used for creating additional waiters.
    Unlike the waiters available directly from botocore these waiters will
    automatically retry on common (temporary) AWS failures.

    This class should be treated as an abstract class and subclassed before use.
    A subclass should:
    - create the necessary client to pass to BaseWaiterFactory.__init__
    - override _BaseWaiterFactory._waiter_model_data to return the data defining
      the waiter

    Usage:
    waiter_factory = BaseWaiterFactory(module, client)
    waiter = waiters.get_waiter('my_waiter_name')
    waiter.wait(**params)
    """

    module = None
    client = None

    def __init__(self, module, client):
        self.module = module
        self.client = client
        # While it would be nice to supliment this with the upstream data,
        # unfortunately client doesn't have a public method for getting the
        # waiter configs.
        data = self._inject_ratelimit_retries(self._waiter_model_data)
        self._model = botocore.waiter.WaiterModel(
            waiter_config=dict(version=2, waiters=data),
        )

    @property
    def _waiter_model_data(self):
        r"""
        Subclasses should override this method to return a dictionary mapping
        waiter names to the waiter definition.

        This data is similar to the data found in botocore's waiters-2.json
        files (for example: botocore/botocore/data/ec2/2016-11-15/waiters-2.json)
        with two differences:
        1) Waiter names do not have transformations applied during lookup
        2) Only the 'waiters' data is required, the data is assumed to be
           version 2

        for example:

        @property
        def _waiter_model_data(self):
            return dict(
                tgw_attachment_deleted=dict(
                    operation='DescribeTransitGatewayAttachments',
                    delay=5, maxAttempts=120,
                    acceptors=[
                        dict(state='retry', matcher='pathAll', expected='deleting', argument='TransitGatewayAttachments[].State'),
                        dict(state='success', matcher='pathAll', expected='deleted', argument='TransitGatewayAttachments[].State'),
                        dict(state='success', matcher='path', expected=True, argument='length(TransitGatewayAttachments[]) == `0`'),
                        dict(state='success', matcher='error', expected='InvalidRouteTableID.NotFound'),
                    ]
                ),
            )

        or

        @property
        def _waiter_model_data(self):
            return {
                "instance_exists": {
                    "delay": 5,
                    "maxAttempts": 40,
                    "operation": "DescribeInstances",
                    "acceptors": [
                        {
                            "matcher": "path",
                            "expected": true,
                            "argument": "length(Reservations[]) > `0`",
                            "state": "success"
                        },
                        {
                            "matcher": "error",
                            "expected": "InvalidInstanceID.NotFound",
                            "state": "retry"
                        }
                    ]
                },
            }
        """

        return dict()

    def _inject_ratelimit_retries(self, model):
        extra_retries = [
            "RequestLimitExceeded",
            "Unavailable",
            "ServiceUnavailable",
            "InternalFailure",
            "InternalError",
            "TooManyRequestsException",
            "Throttling",
        ]

        acceptors = []
        for error in extra_retries:
            acceptors.append(dict(state="retry", matcher="error", expected=error))

        _model = deepcopy(model)
        for waiter in _model:
            _model[waiter]["acceptors"].extend(acceptors)

        return _model

    def get_waiter(self, waiter_name):
        waiters = self._model.waiter_names
        if waiter_name not in waiters:
            self.module.fail_json(f"Unable to find waiter {waiter_name}.  Available_waiters: {waiters}")
        return botocore.waiter.create_waiter_with_client(
            waiter_name,
            self._model,
            self.client,
        )


class Boto3Mixin:
    @staticmethod
    def aws_error_handler(description):
        r"""
        A simple wrapper that handles the usual botocore exceptions and exits
        with module.fail_json_aws.  Designed to be used with BaseResourceManager.
        Assumptions:
          1) First argument (usually `self` of method being wrapped will have a
             'module' attribute which is an AnsibleAWSModule
          2) First argument of method being wrapped will have an
            _extra_error_output() method which takes no arguments and returns a
            dictionary of extra parameters to be returned in the event of a
            botocore exception.
        Parameters:
          description (string): In the event of a botocore exception the error
                                message will be 'Failed to {DESCRIPTION}'.

        Example Usage:
            class ExampleClass(Boto3Mixin):
                def __init__(self, module)
                    self.module = module
                    self._get_client()

                @Boto3Mixin.aws_error_handler("connect to AWS")
                def _get_client(self):
                    self.client = self.module.client('ec2')

                @Boto3Mixin.aws_error_handler("describe EC2 instances")
                def _do_something(**params):
                    return self.client.describe_instances(**params)
        """

        def wrapper(func):
            @wraps(func)
            def handler(_self, *args, **kwargs):
                extra_ouput = _self._extra_error_output()
                try:
                    return func(_self, *args, **kwargs)
                except botocore.exceptions.WaiterError as e:
                    _self.module.fail_json_aws(e, msg=f"Failed waiting for {description}", **extra_ouput)
                except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                    _self.module.fail_json_aws(e, msg=f"Failed to {description}", **extra_ouput)

            return handler

        return wrapper

    def _normalize_boto3_resource(self, resource, add_tags=False):
        r"""
        Performs common boto3 resource to Ansible resource conversion.
        `resource['Tags']` will by default be converted from the boto3 tag list
        format to a simple dictionary.
        Parameters:
          resource (dict): The boto3 style resource to convert to the normal Ansible
                           format (snake_case).
          add_tags (bool): When `true`, if a resource does not have 'Tags' property
                           the returned resource will have tags set to an empty
                           dictionary.
        """
        if resource is None:
            return None

        tags = resource.get("Tags", None)
        if tags:
            tags = boto3_tag_list_to_ansible_dict(tags)
        elif add_tags or tags is not None:
            tags = {}

        normalized_resource = camel_dict_to_snake_dict(resource)
        if tags is not None:
            normalized_resource["tags"] = tags
        return normalized_resource

    def _extra_error_output(self):
        # In the event of an error it can be helpful to ouput things like the
        # 'name'/'arn' of a resource.
        return dict()


class BaseResourceManager(Boto3Mixin):
    def __init__(self, module):
        r"""
        Parameters:
            module (AnsibleAWSModule): An Ansible module.
        """
        self.module = module
        self.changed = False
        self.original_resource = dict()
        self.updated_resource = dict()
        self._resource_updates = dict()
        self._preupdate_resource = dict()
        self._wait = True
        self._wait_timeout = None
        super(BaseResourceManager, self).__init__()

    def _merge_resource_changes(self, filter_immutable=True, creation=False):
        """
        Merges the contents of the 'pre_update' resource and metadata variables
        with the pending updates
        """
        resource = deepcopy(self._preupdate_resource)
        resource.update(self._resource_updates)

        if filter_immutable:
            resource = self._filter_immutable_resource_attributes(resource)

        return resource

    def _filter_immutable_resource_attributes(self, resource):
        return deepcopy(resource)

    def _do_creation_wait(self, **params):
        pass

    def _do_deletion_wait(self, **params):
        pass

    def _do_update_wait(self, **params):
        pass

    @property
    def _waiter_config(self):
        params = dict()
        if self._wait_timeout:
            delay = min(5, self._wait_timeout)
            max_attempts = self._wait_timeout // delay
            config = dict(Delay=delay, MaxAttempts=max_attempts)
            params["WaiterConfig"] = config
        return params

    def _wait_for_deletion(self):
        if not self._wait:
            return
        params = self._waiter_config
        self._do_deletion_wait(**params)

    def _wait_for_creation(self):
        if not self._wait:
            return
        params = self._waiter_config
        self._do_creation_wait(**params)

    def _wait_for_update(self):
        if not self._wait:
            return
        params = self._waiter_config
        self._do_update_wait(**params)

    def _generate_updated_resource(self):
        """
        Merges all pending changes into self.updated_resource
        Used during check mode where it's not possible to get and
        refresh the resource
        """
        return self._merge_resource_changes(filter_immutable=False)

    # If you override _flush_update you're responsible for handling check_mode
    # If you override _do_update_resource you'll only be called if check_mode == False
    def _flush_create(self):
        changed = True

        if not self.module.check_mode:
            changed = self._do_create_resource()
            self._wait_for_creation()
            self._do_creation_wait()
            self.updated_resource = self.get_resource()
        else:  # (CHECK MODE)
            self.updated_resource = self._normalize_resource(self._generate_updated_resource())

        self._resource_updates = dict()
        self.changed = changed
        return True

    def _check_updates_pending(self):
        if self._resource_updates:
            return True
        return False

    # If you override _flush_update you're responsible for handling check_mode
    # If you override _do_update_resource you'll only be called if there are
    # updated pending and check_mode == False
    def _flush_update(self):
        if not self._check_updates_pending():
            self.updated_resource = self.original_resource
            return False

        if not self.module.check_mode:
            self._do_update_resource()
            self._wait_for_update()
            self.updated_resource = self.get_resource()
        else:  # (CHECK_MODE)
            self.updated_resource = self._normalize_resource(self._generate_updated_resource())

        self._resource_updates = dict()
        return True

    def flush_changes(self):
        if self.original_resource:
            return self._flush_update()
        else:
            return self._flush_create()

    def _set_resource_value(self, key, value, description=None, immutable=False):
        if value is None:
            return False
        if value == self._get_resource_value(key):
            return False
        if immutable and self.original_resource:
            if description is None:
                description = key
            self.module.fail_json(msg=f"{description} can not be updated after creation")
        self._resource_updates[key] = value
        self.changed = True
        return True

    def _get_resource_value(self, key, default=None):
        default_value = self._preupdate_resource.get(key, default)
        return self._resource_updates.get(key, default_value)

    def set_wait(self, wait):
        if wait is None:
            return False
        if wait == self._wait:
            return False

        self._wait = wait
        return True

    def set_wait_timeout(self, timeout):
        if timeout is None:
            return False
        if timeout == self._wait_timeout:
            return False

        self._wait_timeout = timeout
        return True



# Copyright (c) 2022, Jonathan Lung <lungj@heresjono.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
name: bitwarden
author:
  - Jonathan Lung (@lungj) <lungj@heresjono.com>
requirements:
  - bw (command line utility)
  - be logged into bitwarden
  - bitwarden vault unlocked
  - E(BW_SESSION) environment variable set
short_description: Retrieve secrets from Bitwarden
version_added: 5.4.0
description:
  - Retrieve secrets from Bitwarden.
options:
  _terms:
    description: Key(s) to fetch values for from login info.
    required: true
    type: list
    elements: str
  search:
    description:
      - Field to retrieve, for example V(name) or V(id).
      - If set to V(id), only zero or one element can be returned. Use the Jinja C(first) filter to get the only list element.
      - If set to V(None) or V(''), or if O(_terms) is empty, records are not filtered by fields.
    type: str
    default: name
    version_added: 5.7.0
  field:
    description: Field to fetch. Leave unset to fetch whole response.
    type: str
  collection_id:
    description:
      - Collection ID to filter results by collection. Leave unset to skip filtering.
      - O(collection_id) and O(collection_name) are mutually exclusive.
    type: str
    version_added: 6.3.0
  collection_name:
    description:
      - Collection name to filter results by collection. Leave unset to skip filtering.
      - O(collection_id) and O(collection_name) are mutually exclusive.
    type: str
    version_added: 10.4.0
  organization_id:
    description: Organization ID to filter results by organization. Leave unset to skip filtering.
    type: str
    version_added: 8.5.0
  bw_session:
    description: Pass session key instead of reading from env.
    type: str
    version_added: 8.4.0
  result_count:
    description:
      - Number of results expected for the lookup query. Task fails if O(result_count) is set but does not match the number
        of query results. Leave empty to skip this check.
    type: int
    version_added: 10.4.0
"""

EXAMPLES = r"""
- name: "Get 'password' from all Bitwarden records named 'a_test'"
  ansible.builtin.debug:
    msg: >-
      {{ lookup('community.general.bitwarden', 'a_test', field='password') }}

- name: "Get 'password' from Bitwarden record with ID 'bafba515-af11-47e6-abe3-af1200cd18b2'"
  ansible.builtin.debug:
    msg: >-
      {{ lookup('community.general.bitwarden', 'bafba515-af11-47e6-abe3-af1200cd18b2', search='id', field='password') | first }}

- name: "Get 'password' from all Bitwarden records named 'a_test' from collection"
  ansible.builtin.debug:
    msg: >-
      {{ lookup('community.general.bitwarden', 'a_test', field='password', collection_id='bafba515-af11-47e6-abe3-af1200cd18b2') }}

- name: "Get list of all full Bitwarden records named 'a_test'"
  ansible.builtin.debug:
    msg: >-
      {{ lookup('community.general.bitwarden', 'a_test') }}

- name: "Get custom field 'api_key' from all Bitwarden records named 'a_test'"
  ansible.builtin.debug:
    msg: >-
      {{ lookup('community.general.bitwarden', 'a_test', field='api_key') }}

- name: "Get 'password' from all Bitwarden records named 'a_test', using given session key"
  ansible.builtin.debug:
    msg: >-
      {{ lookup('community.general.bitwarden', 'a_test', field='password', bw_session='bXZ9B5TXi6...') }}

- name: "Get all Bitwarden records from collection"
  ansible.builtin.debug:
    msg: >-
      {{ lookup('community.general.bitwarden', None, collection_id='bafba515-af11-47e6-abe3-af1200cd18b2') }}

- name: "Get all Bitwarden records from collection"
  ansible.builtin.debug:
    msg: >-
      {{ lookup('community.general.bitwarden', None, collection_name='my_collections/test_collection') }}

- name: "Get Bitwarden record named 'a_test', ensure there is exactly one match"
  ansible.builtin.debug:
    msg: >-
      {{ lookup('community.general.bitwarden', 'a_test', result_count=1) }}
"""

RETURN = r"""
_raw:
  description:
    - A one-element list that contains a list of requested fields or JSON objects of matches.
    - If you use C(query), you get a list of lists. If you use C(lookup) without C(wantlist=true), this always gets reduced
      to a list of field values or JSON objects.
  type: list
  elements: list
"""

from subprocess import PIPE, Popen

from ansible.errors import AnsibleError, AnsibleOptionsError
from ansible.module_utils.common.text.converters import to_bytes, to_text
from ansible.parsing.ajson import AnsibleJSONDecoder
from ansible.plugins.lookup import LookupBase

from ansible_collections.community.general.plugins.plugin_utils._lookup import check_for_wrong_terms


class BitwardenException(AnsibleError):
    pass


class Bitwarden:
    def __init__(self, path="bw"):
        self._cli_path = path
        self._session = None

    @property
    def cli_path(self):
        return self._cli_path

    @property
    def session(self):
        return self._session

    @session.setter
    def session(self, value):
        self._session = value

    @property
    def unlocked(self):
        out, err = self._run(["status"], stdin="")
        decoded = AnsibleJSONDecoder().raw_decode(out)[0]
        return decoded["status"] == "unlocked"

    def _run(self, args, stdin=None, expected_rc=0):
        if self.session:
            args += ["--session", self.session]

        p = Popen([self.cli_path] + args, stdout=PIPE, stderr=PIPE, stdin=PIPE)
        out, err = p.communicate(to_bytes(stdin))
        rc = p.wait()
        if rc != expected_rc:
            if len(args) > 2 and args[0] == "get" and args[1] == "item" and b"Not found." in err:
                return "null", ""
            raise BitwardenException(err)
        return to_text(out, errors="surrogate_or_strict"), to_text(err, errors="surrogate_or_strict")

    def _get_matches(self, search_value, search_field, collection_id=None, organization_id=None):
        """Return matching records whose search_field is equal to key."""

        # Prepare set of params for Bitwarden CLI
        if search_field == "id":
            params = ["get", "item", search_value]
        else:
            params = ["list", "items"]
            if search_value:
                params.extend(["--search", search_value])

        if collection_id:
            params.extend(["--collectionid", collection_id])
        if organization_id:
            params.extend(["--organizationid", organization_id])

        out, err = self._run(params)

        # This includes things that matched in different fields.
        initial_matches = AnsibleJSONDecoder().raw_decode(out)[0]

        if search_field == "id":
            if initial_matches is None:
                initial_matches = []
            else:
                initial_matches = [initial_matches]

        # Filter to only include results from the right field, if a search is requested by value or field
        return [
            item
            for item in initial_matches
            if not search_value or not search_field or item.get(search_field) == search_value
        ]

    def get_field(self, field, search_value, search_field="name", collection_id=None, organization_id=None):
        """Return a list of the specified field for records whose search_field match search_value
        and filtered by collection if collection has been provided.

        If field is None, return the whole record for each match.
        """
        matches = self._get_matches(search_value, search_field, collection_id, organization_id)
        if not field:
            return matches
        field_matches = []
        for match in matches:
            # if there are no custom fields, then `match` has no key 'fields'
            if "fields" in match:
                custom_field_found = False
                for custom_field in match["fields"]:
                    if field == custom_field["name"]:
                        field_matches.append(custom_field["value"])
                        custom_field_found = True
                        break
                if custom_field_found:
                    continue
            if "login" in match and field in match["login"]:
                field_matches.append(match["login"][field])
                continue
            if field in match:
                field_matches.append(match[field])
                continue

        if matches and not field_matches:
            raise AnsibleError(f"field {field} does not exist in {search_value}")

        return field_matches

    def get_collection_ids(self, collection_name: str, organization_id=None) -> list[str]:
        """Return matching IDs of collections whose name is equal to collection_name."""

        # Prepare set of params for Bitwarden CLI
        params = ["list", "collections", "--search", collection_name]

        if organization_id:
            params.extend(["--organizationid", organization_id])

        out, err = self._run(params)

        # This includes things that matched in different fields.
        initial_matches = AnsibleJSONDecoder().raw_decode(out)[0]  # type: ignore[operator]

        # Filter to only return the ID of a collections with exactly matching name
        return [item["id"] for item in initial_matches if str(item.get("name")).lower() == collection_name.lower()]


class LookupModule(LookupBase):
    def run(self, terms=None, variables=None, **kwargs):
        self.set_options(var_options=variables, direct=kwargs)
        check_for_wrong_terms(self, direct=kwargs)
        field = self.get_option("field")
        search_field = self.get_option("search")
        collection_id = self.get_option("collection_id")
        collection_name = self.get_option("collection_name")
        organization_id = self.get_option("organization_id")
        result_count = self.get_option("result_count")
        _bitwarden.session = self.get_option("bw_session")

        if not _bitwarden.unlocked:
            raise AnsibleError("Bitwarden Vault locked. Run 'bw unlock'.")

        if not terms:
            terms = [None]

        if collection_name and collection_id:
            raise AnsibleOptionsError("'collection_name' and 'collection_id' are mutually exclusive!")
        elif collection_name:
            collection_ids = _bitwarden.get_collection_ids(collection_name, organization_id)
            if not collection_ids:
                raise BitwardenException("No matching collections found!")
        else:
            collection_ids = [collection_id]

        results = [
            _bitwarden.get_field(field, term, search_field, collection_id, organization_id)
            for collection_id in collection_ids
            for term in terms
        ]

        for result in results:
            if result_count is not None and len(result) != result_count:
                raise BitwardenException(
                    f"Number of results doesn't match result_count! ({len(result)} != {result_count})"
                )

        return results


_bitwarden = Bitwarden()
